from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import Client
from ...types import UNSET, Response, Unset


def _get_kwargs(
    organization_name: str,
    data_set_name: str,
    data_layer_name: str,
    *,
    client: Client,
    token: Union[Unset, None, str] = UNSET,
    x: int,
    y: int,
    z: int,
    width: int,
    height: int,
    depth: int,
    mag: str,
    half_byte: Union[Unset, None, bool] = False,
    mapping_name: Union[Unset, None, str] = UNSET,
) -> Dict[str, Any]:
    url = "{}/data/datasets/{organizationName}/{dataSetName}/layers/{dataLayerName}/data".format(
        client.base_url,
        organizationName=organization_name,
        dataSetName=data_set_name,
        dataLayerName=data_layer_name,
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    params["token"] = token

    params["x"] = x

    params["y"] = y

    params["z"] = z

    params["width"] = width

    params["height"] = height

    params["depth"] = depth

    params["mag"] = mag

    params["halfByte"] = half_byte

    params["mappingName"] = mapping_name

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "follow_redirects": client.follow_redirects,
        "params": params,
    }


def _parse_response(*, client: Client, response: httpx.Response) -> Optional[Any]:
    if response.status_code == HTTPStatus.OK:
        return None
    if response.status_code == HTTPStatus.BAD_REQUEST:
        return None
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Client, response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    organization_name: str,
    data_set_name: str,
    data_layer_name: str,
    *,
    client: Client,
    token: Union[Unset, None, str] = UNSET,
    x: int,
    y: int,
    z: int,
    width: int,
    height: int,
    depth: int,
    mag: str,
    half_byte: Union[Unset, None, bool] = False,
    mapping_name: Union[Unset, None, str] = UNSET,
) -> Response[Any]:
    """Get raw binary data from a bounding box in a dataset layer

    Args:
        organization_name (str):
        data_set_name (str):
        data_layer_name (str):
        token (Union[Unset, None, str]):
        x (int):
        y (int):
        z (int):
        width (int):
        height (int):
        depth (int):
        mag (str):
        half_byte (Union[Unset, None, bool]):
        mapping_name (Union[Unset, None, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        organization_name=organization_name,
        data_set_name=data_set_name,
        data_layer_name=data_layer_name,
        client=client,
        token=token,
        x=x,
        y=y,
        z=z,
        width=width,
        height=height,
        depth=depth,
        mag=mag,
        half_byte=half_byte,
        mapping_name=mapping_name,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(client=client, response=response)


async def asyncio_detailed(
    organization_name: str,
    data_set_name: str,
    data_layer_name: str,
    *,
    client: Client,
    token: Union[Unset, None, str] = UNSET,
    x: int,
    y: int,
    z: int,
    width: int,
    height: int,
    depth: int,
    mag: str,
    half_byte: Union[Unset, None, bool] = False,
    mapping_name: Union[Unset, None, str] = UNSET,
) -> Response[Any]:
    """Get raw binary data from a bounding box in a dataset layer

    Args:
        organization_name (str):
        data_set_name (str):
        data_layer_name (str):
        token (Union[Unset, None, str]):
        x (int):
        y (int):
        z (int):
        width (int):
        height (int):
        depth (int):
        mag (str):
        half_byte (Union[Unset, None, bool]):
        mapping_name (Union[Unset, None, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        organization_name=organization_name,
        data_set_name=data_set_name,
        data_layer_name=data_layer_name,
        client=client,
        token=token,
        x=x,
        y=y,
        z=z,
        width=width,
        height=height,
        depth=depth,
        mag=mag,
        half_byte=half_byte,
        mapping_name=mapping_name,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(client=client, response=response)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

import attr

if TYPE_CHECKING:
    from ..models.user_logged_time_response_200_logged_time_item_payment_interval import (
        UserLoggedTimeResponse200LoggedTimeItemPaymentInterval,
    )


T = TypeVar("T", bound="UserLoggedTimeResponse200LoggedTimeItem")


@attr.s(auto_attribs=True)
class UserLoggedTimeResponse200LoggedTimeItem:
    """
    Attributes:
        payment_interval (UserLoggedTimeResponse200LoggedTimeItemPaymentInterval):
        duration_in_seconds (int):
    """

    payment_interval: "UserLoggedTimeResponse200LoggedTimeItemPaymentInterval"
    duration_in_seconds: int
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        payment_interval = self.payment_interval.to_dict()

        duration_in_seconds = self.duration_in_seconds

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "paymentInterval": payment_interval,
                "durationInSeconds": duration_in_seconds,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.user_logged_time_response_200_logged_time_item_payment_interval import (
            UserLoggedTimeResponse200LoggedTimeItemPaymentInterval,
        )

        d = src_dict.copy()
        payment_interval = (
            UserLoggedTimeResponse200LoggedTimeItemPaymentInterval.from_dict(
                d.pop("paymentInterval")
            )
        )

        duration_in_seconds = d.pop("durationInSeconds")

        user_logged_time_response_200_logged_time_item = cls(
            payment_interval=payment_interval,
            duration_in_seconds=duration_in_seconds,
        )

        user_logged_time_response_200_logged_time_item.additional_properties = d
        return user_logged_time_response_200_logged_time_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import Client
from ...models.dataset_finish_upload_json_body import DatasetFinishUploadJsonBody
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: Client,
    json_body: DatasetFinishUploadJsonBody,
    token: Union[Unset, None, str] = UNSET,
) -> Dict[str, Any]:
    url = "{}/data/datasets/finishUpload".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    params["token"] = token

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    json_json_body = json_body.to_dict()

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "follow_redirects": client.follow_redirects,
        "json": json_json_body,
        "params": params,
    }


def _parse_response(*, client: Client, response: httpx.Response) -> Optional[Any]:
    if response.status_code == HTTPStatus.OK:
        return None
    if response.status_code == HTTPStatus.BAD_REQUEST:
        return None
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Client, response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Client,
    json_body: DatasetFinishUploadJsonBody,
    token: Union[Unset, None, str] = UNSET,
) -> Response[Any]:
    """Finish dataset upload, call after all chunks have been uploaded via uploadChunk
    Expects:
     - As JSON object body with keys:
      - uploadId (string): upload id that was also used in chunk upload (this time without file paths)
      - organization (string): owning organization name
      - name (string): dataset name
      - needsConversion (boolean): mark as true for non-wkw datasets. They are stored differently and a
    conversion job can later be run.
     - As GET parameter:
      - token (string): datastore token identifying the uploading user

    Args:
        token (Union[Unset, None, str]):
        json_body (DatasetFinishUploadJsonBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
        token=token,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(client=client, response=response)


async def asyncio_detailed(
    *,
    client: Client,
    json_body: DatasetFinishUploadJsonBody,
    token: Union[Unset, None, str] = UNSET,
) -> Response[Any]:
    """Finish dataset upload, call after all chunks have been uploaded via uploadChunk
    Expects:
     - As JSON object body with keys:
      - uploadId (string): upload id that was also used in chunk upload (this time without file paths)
      - organization (string): owning organization name
      - name (string): dataset name
      - needsConversion (boolean): mark as true for non-wkw datasets. They are stored differently and a
    conversion job can later be run.
     - As GET parameter:
      - token (string): datastore token identifying the uploading user

    Args:
        token (Union[Unset, None, str]):
        json_body (DatasetFinishUploadJsonBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
        token=token,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(client=client, response=response)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.project_info_by_id_response_200_owner import (
        ProjectInfoByIdResponse200Owner,
    )


T = TypeVar("T", bound="ProjectInfoByIdResponse200")


@attr.s(auto_attribs=True)
class ProjectInfoByIdResponse200:
    """
    Attributes:
        name (str):
        team (str):
        team_name (str):
        priority (int):
        paused (int):
        expected_time (int):
        id (str):
        created (int):
        owner (Union[Unset, ProjectInfoByIdResponse200Owner]):
        is_blacklisted_from_report (Union[Unset, int]):
    """

    name: str
    team: str
    team_name: str
    priority: int
    paused: int
    expected_time: int
    id: str
    created: int
    owner: Union[Unset, "ProjectInfoByIdResponse200Owner"] = UNSET
    is_blacklisted_from_report: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        team = self.team
        team_name = self.team_name
        priority = self.priority
        paused = self.paused
        expected_time = self.expected_time
        id = self.id
        created = self.created
        owner: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.owner, Unset):
            owner = self.owner.to_dict()

        is_blacklisted_from_report = self.is_blacklisted_from_report

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "team": team,
                "teamName": team_name,
                "priority": priority,
                "paused": paused,
                "expectedTime": expected_time,
                "id": id,
                "created": created,
            }
        )
        if owner is not UNSET:
            field_dict["owner"] = owner
        if is_blacklisted_from_report is not UNSET:
            field_dict["isBlacklistedFromReport"] = is_blacklisted_from_report

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.project_info_by_id_response_200_owner import (
            ProjectInfoByIdResponse200Owner,
        )

        d = src_dict.copy()
        name = d.pop("name")

        team = d.pop("team")

        team_name = d.pop("teamName")

        priority = d.pop("priority")

        paused = d.pop("paused")

        expected_time = d.pop("expectedTime")

        id = d.pop("id")

        created = d.pop("created")

        _owner = d.pop("owner", UNSET)
        owner: Union[Unset, ProjectInfoByIdResponse200Owner]
        if isinstance(_owner, Unset):
            owner = UNSET
        else:
            owner = ProjectInfoByIdResponse200Owner.from_dict(_owner)

        is_blacklisted_from_report = d.pop("isBlacklistedFromReport", UNSET)

        project_info_by_id_response_200 = cls(
            name=name,
            team=team,
            team_name=team_name,
            priority=priority,
            paused=paused,
            expected_time=expected_time,
            id=id,
            created=created,
            owner=owner,
            is_blacklisted_from_report=is_blacklisted_from_report,
        )

        project_info_by_id_response_200.additional_properties = d
        return project_info_by_id_response_200

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.annotation_infos_by_task_id_response_200_item_annotation_layers_item import (
        AnnotationInfosByTaskIdResponse200ItemAnnotationLayersItem,
    )
    from ..models.annotation_infos_by_task_id_response_200_item_data_store import (
        AnnotationInfosByTaskIdResponse200ItemDataStore,
    )
    from ..models.annotation_infos_by_task_id_response_200_item_owner import (
        AnnotationInfosByTaskIdResponse200ItemOwner,
    )
    from ..models.annotation_infos_by_task_id_response_200_item_restrictions import (
        AnnotationInfosByTaskIdResponse200ItemRestrictions,
    )
    from ..models.annotation_infos_by_task_id_response_200_item_settings import (
        AnnotationInfosByTaskIdResponse200ItemSettings,
    )
    from ..models.annotation_infos_by_task_id_response_200_item_stats import (
        AnnotationInfosByTaskIdResponse200ItemStats,
    )
    from ..models.annotation_infos_by_task_id_response_200_item_task import (
        AnnotationInfosByTaskIdResponse200ItemTask,
    )
    from ..models.annotation_infos_by_task_id_response_200_item_tracing_store import (
        AnnotationInfosByTaskIdResponse200ItemTracingStore,
    )
    from ..models.annotation_infos_by_task_id_response_200_item_user import (
        AnnotationInfosByTaskIdResponse200ItemUser,
    )


T = TypeVar("T", bound="AnnotationInfosByTaskIdResponse200Item")


@attr.s(auto_attribs=True)
class AnnotationInfosByTaskIdResponse200Item:
    """
    Attributes:
        modified (int):
        state (str):
        id (str):
        name (str):
        description (str):
        typ (str):
        organization (str):
        data_store (AnnotationInfosByTaskIdResponse200ItemDataStore):
        tags (List[str]):
        view_configuration (Union[Unset, str]):
        task (Union[Unset, None, AnnotationInfosByTaskIdResponse200ItemTask]):
        stats (Union[Unset, AnnotationInfosByTaskIdResponse200ItemStats]):
        restrictions (Union[Unset, AnnotationInfosByTaskIdResponse200ItemRestrictions]):
        formatted_hash (Union[Unset, str]):
        annotation_layers (Union[Unset, List['AnnotationInfosByTaskIdResponse200ItemAnnotationLayersItem']]):
        data_set_name (Union[Unset, str]):
        tracing_store (Union[Unset, AnnotationInfosByTaskIdResponse200ItemTracingStore]):
        visibility (Union[Unset, str]):
        settings (Union[Unset, AnnotationInfosByTaskIdResponse200ItemSettings]):
        tracing_time (Union[Unset, None, int]):
        teams (Union[Unset, List[Any]]):
        user (Union[Unset, AnnotationInfosByTaskIdResponse200ItemUser]):
        owner (Union[Unset, AnnotationInfosByTaskIdResponse200ItemOwner]):
        contributors (Union[Unset, List[Any]]):
        others_may_edit (Union[Unset, int]):
    """

    modified: int
    state: str
    id: str
    name: str
    description: str
    typ: str
    organization: str
    data_store: "AnnotationInfosByTaskIdResponse200ItemDataStore"
    tags: List[str]
    view_configuration: Union[Unset, str] = UNSET
    task: Union[Unset, None, "AnnotationInfosByTaskIdResponse200ItemTask"] = UNSET
    stats: Union[Unset, "AnnotationInfosByTaskIdResponse200ItemStats"] = UNSET
    restrictions: Union[
        Unset, "AnnotationInfosByTaskIdResponse200ItemRestrictions"
    ] = UNSET
    formatted_hash: Union[Unset, str] = UNSET
    annotation_layers: Union[
        Unset, List["AnnotationInfosByTaskIdResponse200ItemAnnotationLayersItem"]
    ] = UNSET
    data_set_name: Union[Unset, str] = UNSET
    tracing_store: Union[
        Unset, "AnnotationInfosByTaskIdResponse200ItemTracingStore"
    ] = UNSET
    visibility: Union[Unset, str] = UNSET
    settings: Union[Unset, "AnnotationInfosByTaskIdResponse200ItemSettings"] = UNSET
    tracing_time: Union[Unset, None, int] = UNSET
    teams: Union[Unset, List[Any]] = UNSET
    user: Union[Unset, "AnnotationInfosByTaskIdResponse200ItemUser"] = UNSET
    owner: Union[Unset, "AnnotationInfosByTaskIdResponse200ItemOwner"] = UNSET
    contributors: Union[Unset, List[Any]] = UNSET
    others_may_edit: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        modified = self.modified
        state = self.state
        id = self.id
        name = self.name
        description = self.description
        typ = self.typ
        organization = self.organization
        data_store = self.data_store.to_dict()

        tags = self.tags

        view_configuration = self.view_configuration
        task: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.task, Unset):
            task = self.task.to_dict() if self.task else None

        stats: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.stats, Unset):
            stats = self.stats.to_dict()

        restrictions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.restrictions, Unset):
            restrictions = self.restrictions.to_dict()

        formatted_hash = self.formatted_hash
        annotation_layers: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.annotation_layers, Unset):
            annotation_layers = []
            for annotation_layers_item_data in self.annotation_layers:
                annotation_layers_item = annotation_layers_item_data.to_dict()

                annotation_layers.append(annotation_layers_item)

        data_set_name = self.data_set_name
        tracing_store: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.tracing_store, Unset):
            tracing_store = self.tracing_store.to_dict()

        visibility = self.visibility
        settings: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.settings, Unset):
            settings = self.settings.to_dict()

        tracing_time = self.tracing_time
        teams: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.teams, Unset):
            teams = self.teams

        user: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.user, Unset):
            user = self.user.to_dict()

        owner: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.owner, Unset):
            owner = self.owner.to_dict()

        contributors: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.contributors, Unset):
            contributors = self.contributors

        others_may_edit = self.others_may_edit

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "modified": modified,
                "state": state,
                "id": id,
                "name": name,
                "description": description,
                "typ": typ,
                "organization": organization,
                "dataStore": data_store,
                "tags": tags,
            }
        )
        if view_configuration is not UNSET:
            field_dict["viewConfiguration"] = view_configuration
        if task is not UNSET:
            field_dict["task"] = task
        if stats is not UNSET:
            field_dict["stats"] = stats
        if restrictions is not UNSET:
            field_dict["restrictions"] = restrictions
        if formatted_hash is not UNSET:
            field_dict["formattedHash"] = formatted_hash
        if annotation_layers is not UNSET:
            field_dict["annotationLayers"] = annotation_layers
        if data_set_name is not UNSET:
            field_dict["dataSetName"] = data_set_name
        if tracing_store is not UNSET:
            field_dict["tracingStore"] = tracing_store
        if visibility is not UNSET:
            field_dict["visibility"] = visibility
        if settings is not UNSET:
            field_dict["settings"] = settings
        if tracing_time is not UNSET:
            field_dict["tracingTime"] = tracing_time
        if teams is not UNSET:
            field_dict["teams"] = teams
        if user is not UNSET:
            field_dict["user"] = user
        if owner is not UNSET:
            field_dict["owner"] = owner
        if contributors is not UNSET:
            field_dict["contributors"] = contributors
        if others_may_edit is not UNSET:
            field_dict["othersMayEdit"] = others_may_edit

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.annotation_infos_by_task_id_response_200_item_annotation_layers_item import (
            AnnotationInfosByTaskIdResponse200ItemAnnotationLayersItem,
        )
        from ..models.annotation_infos_by_task_id_response_200_item_data_store import (
            AnnotationInfosByTaskIdResponse200ItemDataStore,
        )
        from ..models.annotation_infos_by_task_id_response_200_item_owner import (
            AnnotationInfosByTaskIdResponse200ItemOwner,
        )
        from ..models.annotation_infos_by_task_id_response_200_item_restrictions import (
            AnnotationInfosByTaskIdResponse200ItemRestrictions,
        )
        from ..models.annotation_infos_by_task_id_response_200_item_settings import (
            AnnotationInfosByTaskIdResponse200ItemSettings,
        )
        from ..models.annotation_infos_by_task_id_response_200_item_stats import (
            AnnotationInfosByTaskIdResponse200ItemStats,
        )
        from ..models.annotation_infos_by_task_id_response_200_item_task import (
            AnnotationInfosByTaskIdResponse200ItemTask,
        )
        from ..models.annotation_infos_by_task_id_response_200_item_tracing_store import (
            AnnotationInfosByTaskIdResponse200ItemTracingStore,
        )
        from ..models.annotation_infos_by_task_id_response_200_item_user import (
            AnnotationInfosByTaskIdResponse200ItemUser,
        )

        d = src_dict.copy()
        modified = d.pop("modified")

        state = d.pop("state")

        id = d.pop("id")

        name = d.pop("name")

        description = d.pop("description")

        typ = d.pop("typ")

        organization = d.pop("organization")

        data_store = AnnotationInfosByTaskIdResponse200ItemDataStore.from_dict(
            d.pop("dataStore")
        )

        tags = cast(List[str], d.pop("tags"))

        view_configuration = d.pop("viewConfiguration", UNSET)

        _task = d.pop("task", UNSET)
        task: Union[Unset, None, AnnotationInfosByTaskIdResponse200ItemTask]
        if _task is None:
            task = None
        elif isinstance(_task, Unset):
            task = UNSET
        else:
            task = AnnotationInfosByTaskIdResponse200ItemTask.from_dict(_task)

        _stats = d.pop("stats", UNSET)
        stats: Union[Unset, AnnotationInfosByTaskIdResponse200ItemStats]
        if isinstance(_stats, Unset):
            stats = UNSET
        else:
            stats = AnnotationInfosByTaskIdResponse200ItemStats.from_dict(_stats)

        _restrictions = d.pop("restrictions", UNSET)
        restrictions: Union[Unset, AnnotationInfosByTaskIdResponse200ItemRestrictions]
        if isinstance(_restrictions, Unset):
            restrictions = UNSET
        else:
            restrictions = AnnotationInfosByTaskIdResponse200ItemRestrictions.from_dict(
                _restrictions
            )

        formatted_hash = d.pop("formattedHash", UNSET)

        annotation_layers = []
        _annotation_layers = d.pop("annotationLayers", UNSET)
        for annotation_layers_item_data in _annotation_layers or []:
            annotation_layers_item = (
                AnnotationInfosByTaskIdResponse200ItemAnnotationLayersItem.from_dict(
                    annotation_layers_item_data
                )
            )

            annotation_layers.append(annotation_layers_item)

        data_set_name = d.pop("dataSetName", UNSET)

        _tracing_store = d.pop("tracingStore", UNSET)
        tracing_store: Union[Unset, AnnotationInfosByTaskIdResponse200ItemTracingStore]
        if isinstance(_tracing_store, Unset):
            tracing_store = UNSET
        else:
            tracing_store = (
                AnnotationInfosByTaskIdResponse200ItemTracingStore.from_dict(
                    _tracing_store
                )
            )

        visibility = d.pop("visibility", UNSET)

        _settings = d.pop("settings", UNSET)
        settings: Union[Unset, AnnotationInfosByTaskIdResponse200ItemSettings]
        if isinstance(_settings, Unset):
            settings = UNSET
        else:
            settings = AnnotationInfosByTaskIdResponse200ItemSettings.from_dict(
                _settings
            )

        tracing_time = d.pop("tracingTime", UNSET)

        teams = cast(List[Any], d.pop("teams", UNSET))

        _user = d.pop("user", UNSET)
        user: Union[Unset, AnnotationInfosByTaskIdResponse200ItemUser]
        if isinstance(_user, Unset):
            user = UNSET
        else:
            user = AnnotationInfosByTaskIdResponse200ItemUser.from_dict(_user)

        _owner = d.pop("owner", UNSET)
        owner: Union[Unset, AnnotationInfosByTaskIdResponse200ItemOwner]
        if isinstance(_owner, Unset):
            owner = UNSET
        else:
            owner = AnnotationInfosByTaskIdResponse200ItemOwner.from_dict(_owner)

        contributors = cast(List[Any], d.pop("contributors", UNSET))

        others_may_edit = d.pop("othersMayEdit", UNSET)

        annotation_infos_by_task_id_response_200_item = cls(
            modified=modified,
            state=state,
            id=id,
            name=name,
            description=description,
            typ=typ,
            organization=organization,
            data_store=data_store,
            tags=tags,
            view_configuration=view_configuration,
            task=task,
            stats=stats,
            restrictions=restrictions,
            formatted_hash=formatted_hash,
            annotation_layers=annotation_layers,
            data_set_name=data_set_name,
            tracing_store=tracing_store,
            visibility=visibility,
            settings=settings,
            tracing_time=tracing_time,
            teams=teams,
            user=user,
            owner=owner,
            contributors=contributors,
            others_may_edit=others_may_edit,
        )

        annotation_infos_by_task_id_response_200_item.additional_properties = d
        return annotation_infos_by_task_id_response_200_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import Client
from ...types import UNSET, Response, Unset


def _get_kwargs(
    typ: str,
    id: str,
    *,
    client: Client,
    skeleton_version: Union[Unset, None, int] = UNSET,
    volume_version: Union[Unset, None, int] = UNSET,
    skip_volume_data: Union[Unset, None, bool] = UNSET,
) -> Dict[str, Any]:
    url = "{}/api/annotations/{typ}/{id}/download".format(
        client.base_url, typ=typ, id=id
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    params["skeletonVersion"] = skeleton_version

    params["volumeVersion"] = volume_version

    params["skipVolumeData"] = skip_volume_data

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "follow_redirects": client.follow_redirects,
        "params": params,
    }


def _parse_response(*, client: Client, response: httpx.Response) -> Optional[Any]:
    if response.status_code == HTTPStatus.OK:
        return None
    if response.status_code == HTTPStatus.BAD_REQUEST:
        return None
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Client, response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    typ: str,
    id: str,
    *,
    client: Client,
    skeleton_version: Union[Unset, None, int] = UNSET,
    volume_version: Union[Unset, None, int] = UNSET,
    skip_volume_data: Union[Unset, None, bool] = UNSET,
) -> Response[Any]:
    """Download an annotation as NML/ZIP

    Args:
        typ (str):
        id (str):
        skeleton_version (Union[Unset, None, int]):
        volume_version (Union[Unset, None, int]):
        skip_volume_data (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        typ=typ,
        id=id,
        client=client,
        skeleton_version=skeleton_version,
        volume_version=volume_version,
        skip_volume_data=skip_volume_data,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(client=client, response=response)


async def asyncio_detailed(
    typ: str,
    id: str,
    *,
    client: Client,
    skeleton_version: Union[Unset, None, int] = UNSET,
    volume_version: Union[Unset, None, int] = UNSET,
    skip_volume_data: Union[Unset, None, bool] = UNSET,
) -> Response[Any]:
    """Download an annotation as NML/ZIP

    Args:
        typ (str):
        id (str):
        skeleton_version (Union[Unset, None, int]):
        volume_version (Union[Unset, None, int]):
        skip_volume_data (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        typ=typ,
        id=id,
        client=client,
        skeleton_version=skeleton_version,
        volume_version=volume_version,
        skip_volume_data=skip_volume_data,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(client=client, response=response)



import warnings
from typing import TYPE_CHECKING, List, Union

import attr

from ..client._generated.api.default import (
    project_info_by_id,
    project_info_by_name,
    task_infos_by_project_id,
)
from ..client._generated.types import Unset
from ..client.context import _get_generated_client
from .user import User

if TYPE_CHECKING:
    from ..client._generated.models.project_info_by_id_response_200 import (
        ProjectInfoByIdResponse200,
    )
    from ..client._generated.models.project_info_by_name_response_200 import (
        ProjectInfoByNameResponse200,
    )
    from .task import Task


@attr.frozen
class Project:
    """Data class containing information about a WEBKNOSSOS project"""

    project_id: str
    name: str
    team_id: str
    team_name: str
    owner_id: str
    priority: int
    paused: bool
    expected_time: int

    @classmethod
    def get_by_id(
        cls, project_id: str
    ) -> "Project":  # pylint: disable=redefined-builtin
        """Returns the project specified by the passed id if your token authorizes you to see it."""
        client = _get_generated_client(enforce_auth=True)
        response = project_info_by_id.sync(project_id, client=client)
        assert response is not None, "Could not fetch project by id."
        return cls._from_generated_response(response)

    @classmethod
    def get_by_name(cls, name: str) -> "Project":
        """Returns the user specified by the passed name if your token authorizes you to see it."""
        client = _get_generated_client(enforce_auth=True)
        response = project_info_by_name.sync(name, client=client)
        assert response is not None, "Could not fetch project by name."
        return cls._from_generated_response(response)

    def get_tasks(self, fetch_all: bool = False) -> List["Task"]:
        """Returns the tasks of this project.
        Note: will fetch only the first 1000 entries by default, warns if that means some are missing.
        set parameter pass fetch_all=True to use pagination to fetch all tasks iteratively with pagination.
        """

        from .task import Task

        PAGINATION_LIMIT = 1000
        pagination_page = 0

        client = _get_generated_client(enforce_auth=True)
        response_raw = task_infos_by_project_id.sync_detailed(
            self.project_id,
            limit=PAGINATION_LIMIT,
            page_number=pagination_page,
            include_total_count=True,
            client=client,
        )
        total_count_raw = response_raw.headers.get("X-Total-Count")
        assert total_count_raw is not None, "X-Total-Count header missing from response"
        total_count = int(total_count_raw)
        response = response_raw.parsed
        assert response is not None, "Could not fetch task infos by project id."
        all_tasks = [Task._from_generated_response(t) for t in response]
        if total_count > PAGINATION_LIMIT:
            if fetch_all:
                while total_count > len(all_tasks):
                    pagination_page += 1
                    response = task_infos_by_project_id.sync(
                        self.project_id,
                        limit=PAGINATION_LIMIT,
                        page_number=pagination_page,
                        include_total_count=False,
                        client=client,
                    )
                    assert (
                        response is not None
                    ), "Could not fetch task infos by project id."
                    new_tasks = [Task._from_generated_response(t) for t in response]
                    all_tasks.extend(new_tasks)

            else:
                warnings.warn(
                    f"[INFO] Fetched only {PAGINATION_LIMIT} of {total_count} tasks. Pass fetch_all=True to fetch all tasks iteratively (may be slow!)"
                )

        return all_tasks

    def get_owner(self) -> User:
        """Returns the user that is the owner of this task"""
        return User.get_by_id(self.owner_id)

    @classmethod
    def _from_generated_response(
        cls,
        response: Union["ProjectInfoByIdResponse200", "ProjectInfoByNameResponse200"],
    ) -> "Project":
        assert not isinstance(response.owner, Unset)
        return cls(
            response.id,
            response.name,
            response.team,
            response.team_name,
            response.owner.id,
            response.priority,
            bool(response.paused),
            response.expected_time,
        )



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="LinkedLayerIdentifier")


@attr.s(auto_attribs=True)
class LinkedLayerIdentifier:
    """
    Attributes:
        organization_name (str):
        data_set_name (str):
        layer_name (str):
        new_layer_name (Union[Unset, str]):
    """

    organization_name: str
    data_set_name: str
    layer_name: str
    new_layer_name: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        organization_name = self.organization_name
        data_set_name = self.data_set_name
        layer_name = self.layer_name
        new_layer_name = self.new_layer_name

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "organizationName": organization_name,
                "dataSetName": data_set_name,
                "layerName": layer_name,
            }
        )
        if new_layer_name is not UNSET:
            field_dict["newLayerName"] = new_layer_name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        organization_name = d.pop("organizationName")

        data_set_name = d.pop("dataSetName")

        layer_name = d.pop("layerName")

        new_layer_name = d.pop("newLayerName", UNSET)

        linked_layer_identifier = cls(
            organization_name=organization_name,
            data_set_name=data_set_name,
            layer_name=layer_name,
            new_layer_name=new_layer_name,
        )

        linked_layer_identifier.additional_properties = d
        return linked_layer_identifier

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="BuildInfoResponse200Webknossos")


@attr.s(auto_attribs=True)
class BuildInfoResponse200Webknossos:
    """
    Attributes:
        name (str):
        ci_tag (Union[Unset, str]):
        commit_hash (Union[Unset, str]):
        ci_build (Union[Unset, str]):
        scala_version (Union[Unset, str]):
        version (Union[Unset, str]):
        sbt_version (Union[Unset, str]):
        datastore_api_version (Union[Unset, str]):
        commit_date (Union[Unset, str]):
    """

    name: str
    ci_tag: Union[Unset, str] = UNSET
    commit_hash: Union[Unset, str] = UNSET
    ci_build: Union[Unset, str] = UNSET
    scala_version: Union[Unset, str] = UNSET
    version: Union[Unset, str] = UNSET
    sbt_version: Union[Unset, str] = UNSET
    datastore_api_version: Union[Unset, str] = UNSET
    commit_date: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        ci_tag = self.ci_tag
        commit_hash = self.commit_hash
        ci_build = self.ci_build
        scala_version = self.scala_version
        version = self.version
        sbt_version = self.sbt_version
        datastore_api_version = self.datastore_api_version
        commit_date = self.commit_date

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
            }
        )
        if ci_tag is not UNSET:
            field_dict["ciTag"] = ci_tag
        if commit_hash is not UNSET:
            field_dict["commitHash"] = commit_hash
        if ci_build is not UNSET:
            field_dict["ciBuild"] = ci_build
        if scala_version is not UNSET:
            field_dict["scalaVersion"] = scala_version
        if version is not UNSET:
            field_dict["version"] = version
        if sbt_version is not UNSET:
            field_dict["sbtVersion"] = sbt_version
        if datastore_api_version is not UNSET:
            field_dict["datastoreApiVersion"] = datastore_api_version
        if commit_date is not UNSET:
            field_dict["commitDate"] = commit_date

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        ci_tag = d.pop("ciTag", UNSET)

        commit_hash = d.pop("commitHash", UNSET)

        ci_build = d.pop("ciBuild", UNSET)

        scala_version = d.pop("scalaVersion", UNSET)

        version = d.pop("version", UNSET)

        sbt_version = d.pop("sbtVersion", UNSET)

        datastore_api_version = d.pop("datastoreApiVersion", UNSET)

        commit_date = d.pop("commitDate", UNSET)

        build_info_response_200_webknossos = cls(
            name=name,
            ci_tag=ci_tag,
            commit_hash=commit_hash,
            ci_build=ci_build,
            scala_version=scala_version,
            version=version,
            sbt_version=sbt_version,
            datastore_api_version=datastore_api_version,
            commit_date=commit_date,
        )

        build_info_response_200_webknossos.additional_properties = d
        return build_info_response_200_webknossos

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.task_info_response_200_needed_experience import (
        TaskInfoResponse200NeededExperience,
    )
    from ..models.task_info_response_200_status import TaskInfoResponse200Status
    from ..models.task_info_response_200_type import TaskInfoResponse200Type


T = TypeVar("T", bound="TaskInfoResponse200")


@attr.s(auto_attribs=True)
class TaskInfoResponse200:
    """
    Attributes:
        id (str):
        project_id (str):
        team (str):
        type (TaskInfoResponse200Type):
        data_set (str):
        created (int):
        status (TaskInfoResponse200Status):
        bounding_box (str):
        formatted_hash (Union[Unset, str]):
        project_name (Union[Unset, str]):
        needed_experience (Union[Unset, TaskInfoResponse200NeededExperience]):
        script (Union[Unset, str]):
        tracing_time (Union[Unset, None, int]):
        creation_info (Union[Unset, str]):
        edit_position (Union[Unset, List[int]]):
        edit_rotation (Union[Unset, List[int]]):
    """

    id: str
    project_id: str
    team: str
    type: "TaskInfoResponse200Type"
    data_set: str
    created: int
    status: "TaskInfoResponse200Status"
    bounding_box: str
    formatted_hash: Union[Unset, str] = UNSET
    project_name: Union[Unset, str] = UNSET
    needed_experience: Union[Unset, "TaskInfoResponse200NeededExperience"] = UNSET
    script: Union[Unset, str] = UNSET
    tracing_time: Union[Unset, None, int] = UNSET
    creation_info: Union[Unset, str] = UNSET
    edit_position: Union[Unset, List[int]] = UNSET
    edit_rotation: Union[Unset, List[int]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        project_id = self.project_id
        team = self.team
        type = self.type.to_dict()

        data_set = self.data_set
        created = self.created
        status = self.status.to_dict()

        bounding_box = self.bounding_box
        formatted_hash = self.formatted_hash
        project_name = self.project_name
        needed_experience: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.needed_experience, Unset):
            needed_experience = self.needed_experience.to_dict()

        script = self.script
        tracing_time = self.tracing_time
        creation_info = self.creation_info
        edit_position: Union[Unset, List[int]] = UNSET
        if not isinstance(self.edit_position, Unset):
            edit_position = self.edit_position

        edit_rotation: Union[Unset, List[int]] = UNSET
        if not isinstance(self.edit_rotation, Unset):
            edit_rotation = self.edit_rotation

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "projectId": project_id,
                "team": team,
                "type": type,
                "dataSet": data_set,
                "created": created,
                "status": status,
                "boundingBox": bounding_box,
            }
        )
        if formatted_hash is not UNSET:
            field_dict["formattedHash"] = formatted_hash
        if project_name is not UNSET:
            field_dict["projectName"] = project_name
        if needed_experience is not UNSET:
            field_dict["neededExperience"] = needed_experience
        if script is not UNSET:
            field_dict["script"] = script
        if tracing_time is not UNSET:
            field_dict["tracingTime"] = tracing_time
        if creation_info is not UNSET:
            field_dict["creationInfo"] = creation_info
        if edit_position is not UNSET:
            field_dict["editPosition"] = edit_position
        if edit_rotation is not UNSET:
            field_dict["editRotation"] = edit_rotation

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.task_info_response_200_needed_experience import (
            TaskInfoResponse200NeededExperience,
        )
        from ..models.task_info_response_200_status import TaskInfoResponse200Status
        from ..models.task_info_response_200_type import TaskInfoResponse200Type

        d = src_dict.copy()
        id = d.pop("id")

        project_id = d.pop("projectId")

        team = d.pop("team")

        type = TaskInfoResponse200Type.from_dict(d.pop("type"))

        data_set = d.pop("dataSet")

        created = d.pop("created")

        status = TaskInfoResponse200Status.from_dict(d.pop("status"))

        bounding_box = d.pop("boundingBox")

        formatted_hash = d.pop("formattedHash", UNSET)

        project_name = d.pop("projectName", UNSET)

        _needed_experience = d.pop("neededExperience", UNSET)
        needed_experience: Union[Unset, TaskInfoResponse200NeededExperience]
        if isinstance(_needed_experience, Unset):
            needed_experience = UNSET
        else:
            needed_experience = TaskInfoResponse200NeededExperience.from_dict(
                _needed_experience
            )

        script = d.pop("script", UNSET)

        tracing_time = d.pop("tracingTime", UNSET)

        creation_info = d.pop("creationInfo", UNSET)

        edit_position = cast(List[int], d.pop("editPosition", UNSET))

        edit_rotation = cast(List[int], d.pop("editRotation", UNSET))

        task_info_response_200 = cls(
            id=id,
            project_id=project_id,
            team=team,
            type=type,
            data_set=data_set,
            created=created,
            status=status,
            bounding_box=bounding_box,
            formatted_hash=formatted_hash,
            project_name=project_name,
            needed_experience=needed_experience,
            script=script,
            tracing_time=tracing_time,
            creation_info=creation_info,
            edit_position=edit_position,
            edit_rotation=edit_rotation,
        )

        task_info_response_200.additional_properties = d
        return task_info_response_200

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.instant import Instant


T = TypeVar("T", bound="AnnotationPrivateLinkParams")


@attr.s(auto_attribs=True)
class AnnotationPrivateLinkParams:
    """
    Attributes:
        annotation (str):
        expiration_date_time (Union[Unset, Instant]):
    """

    annotation: str
    expiration_date_time: Union[Unset, "Instant"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        annotation = self.annotation
        expiration_date_time: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.expiration_date_time, Unset):
            expiration_date_time = self.expiration_date_time.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "annotation": annotation,
            }
        )
        if expiration_date_time is not UNSET:
            field_dict["expirationDateTime"] = expiration_date_time

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.instant import Instant

        d = src_dict.copy()
        annotation = d.pop("annotation")

        _expiration_date_time = d.pop("expirationDateTime", UNSET)
        expiration_date_time: Union[Unset, Instant]
        if isinstance(_expiration_date_time, Unset):
            expiration_date_time = UNSET
        else:
            expiration_date_time = Instant.from_dict(_expiration_date_time)

        annotation_private_link_params = cls(
            annotation=annotation,
            expiration_date_time=expiration_date_time,
        )

        annotation_private_link_params.additional_properties = d
        return annotation_private_link_params

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.dataset_list_response_200_item_data_source_id import (
        DatasetListResponse200ItemDataSourceId,
    )


T = TypeVar("T", bound="DatasetListResponse200ItemDataSource")


@attr.s(auto_attribs=True)
class DatasetListResponse200ItemDataSource:
    """
    Attributes:
        id (DatasetListResponse200ItemDataSourceId):
        status (Union[Unset, str]):
    """

    id: "DatasetListResponse200ItemDataSourceId"
    status: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id.to_dict()

        status = self.status

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
            }
        )
        if status is not UNSET:
            field_dict["status"] = status

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.dataset_list_response_200_item_data_source_id import (
            DatasetListResponse200ItemDataSourceId,
        )

        d = src_dict.copy()
        id = DatasetListResponse200ItemDataSourceId.from_dict(d.pop("id"))

        status = d.pop("status", UNSET)

        dataset_list_response_200_item_data_source = cls(
            id=id,
            status=status,
        )

        dataset_list_response_200_item_data_source.additional_properties = d
        return dataset_list_response_200_item_data_source

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Optional, Union

import attr

from ..client._generated.types import Unset
from ..utils import warn_deprecated
from .annotation import Annotation, AnnotationState, AnnotationType

if TYPE_CHECKING:
    from ..client._generated.models.annotation_info_response_200 import (
        AnnotationInfoResponse200,
    )
    from ..client._generated.models.annotation_infos_by_task_id_response_200_item import (
        AnnotationInfosByTaskIdResponse200Item,
    )


@attr.frozen
class AnnotationInfo:
    """Data class containing information about a WEBKNOSSOS annotation"""

    id: str
    owner_id: Optional[str]
    name: str
    description: str
    type: AnnotationType
    state: AnnotationState
    duration_in_seconds: Optional[int]
    modified: Optional[int]

    def download_annotation(self) -> Annotation:
        """Downloads and returns the annotation that is discribed by this AnnotationInfo object"""
        return Annotation.download(self.id)

    @classmethod
    def _from_generated_response(
        cls,
        response: Union[
            "AnnotationInfoResponse200", "AnnotationInfosByTaskIdResponse200Item"
        ],
    ) -> "AnnotationInfo":
        maybe_owner = response.owner or response.user
        owner_id = None if isinstance(maybe_owner, Unset) else maybe_owner.id
        return AnnotationInfo(
            id=response.id,
            owner_id=owner_id,
            name=response.name,
            description=response.description,
            type=AnnotationType(response.typ),
            state=AnnotationState(response.state),
            duration_in_seconds=response.tracing_time // 1000
            if response.tracing_time is not None
            and not isinstance(response.tracing_time, Unset)
            else None,
            modified=response.modified,
        )

    @property
    def user_id(self) -> Optional[str]:
        warn_deprecated("user_id", "owner_id")
        return self.owner_id



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.dataset_info_response_200_data_source_data_layers_item import (
        DatasetInfoResponse200DataSourceDataLayersItem,
    )
    from ..models.dataset_info_response_200_data_source_id import (
        DatasetInfoResponse200DataSourceId,
    )


T = TypeVar("T", bound="DatasetInfoResponse200DataSource")


@attr.s(auto_attribs=True)
class DatasetInfoResponse200DataSource:
    """
    Attributes:
        id (DatasetInfoResponse200DataSourceId):
        data_layers (Union[Unset, List['DatasetInfoResponse200DataSourceDataLayersItem']]):
        scale (Union[Unset, List[float]]):
    """

    id: "DatasetInfoResponse200DataSourceId"
    data_layers: Union[
        Unset, List["DatasetInfoResponse200DataSourceDataLayersItem"]
    ] = UNSET
    scale: Union[Unset, List[float]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id.to_dict()

        data_layers: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.data_layers, Unset):
            data_layers = []
            for data_layers_item_data in self.data_layers:
                data_layers_item = data_layers_item_data.to_dict()

                data_layers.append(data_layers_item)

        scale: Union[Unset, List[float]] = UNSET
        if not isinstance(self.scale, Unset):
            scale = self.scale

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
            }
        )
        if data_layers is not UNSET:
            field_dict["dataLayers"] = data_layers
        if scale is not UNSET:
            field_dict["scale"] = scale

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.dataset_info_response_200_data_source_data_layers_item import (
            DatasetInfoResponse200DataSourceDataLayersItem,
        )
        from ..models.dataset_info_response_200_data_source_id import (
            DatasetInfoResponse200DataSourceId,
        )

        d = src_dict.copy()
        id = DatasetInfoResponse200DataSourceId.from_dict(d.pop("id"))

        data_layers = []
        _data_layers = d.pop("dataLayers", UNSET)
        for data_layers_item_data in _data_layers or []:
            data_layers_item = DatasetInfoResponse200DataSourceDataLayersItem.from_dict(
                data_layers_item_data
            )

            data_layers.append(data_layers_item)

        scale = cast(List[float], d.pop("scale", UNSET))

        dataset_info_response_200_data_source = cls(
            id=id,
            data_layers=data_layers,
            scale=scale,
        )

        dataset_info_response_200_data_source.additional_properties = d
        return dataset_info_response_200_data_source

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar

import attr

T = TypeVar("T", bound="AnnotationInfoResponse200TaskStatus")


@attr.s(auto_attribs=True)
class AnnotationInfoResponse200TaskStatus:
    """
    Attributes:
        pending (int):
        active (int):
        finished (int):
    """

    pending: int
    active: int
    finished: int
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        pending = self.pending
        active = self.active
        finished = self.finished

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "pending": pending,
                "active": active,
                "finished": finished,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        pending = d.pop("pending")

        active = d.pop("active")

        finished = d.pop("finished")

        annotation_info_response_200_task_status = cls(
            pending=pending,
            active=active,
            finished=finished,
        )

        annotation_info_response_200_task_status.additional_properties = d
        return annotation_info_response_200_task_status

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="AnnotationInfoResponse200Restrictions")


@attr.s(auto_attribs=True)
class AnnotationInfoResponse200Restrictions:
    """
    Attributes:
        allow_access (Union[Unset, int]):
        allow_update (Union[Unset, int]):
        allow_finish (Union[Unset, int]):
        allow_download (Union[Unset, int]):
    """

    allow_access: Union[Unset, int] = UNSET
    allow_update: Union[Unset, int] = UNSET
    allow_finish: Union[Unset, int] = UNSET
    allow_download: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        allow_access = self.allow_access
        allow_update = self.allow_update
        allow_finish = self.allow_finish
        allow_download = self.allow_download

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if allow_access is not UNSET:
            field_dict["allowAccess"] = allow_access
        if allow_update is not UNSET:
            field_dict["allowUpdate"] = allow_update
        if allow_finish is not UNSET:
            field_dict["allowFinish"] = allow_finish
        if allow_download is not UNSET:
            field_dict["allowDownload"] = allow_download

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        allow_access = d.pop("allowAccess", UNSET)

        allow_update = d.pop("allowUpdate", UNSET)

        allow_finish = d.pop("allowFinish", UNSET)

        allow_download = d.pop("allowDownload", UNSET)

        annotation_info_response_200_restrictions = cls(
            allow_access=allow_access,
            allow_update=allow_update,
            allow_finish=allow_finish,
            allow_download=allow_download,
        )

        annotation_info_response_200_restrictions.additional_properties = d
        return annotation_info_response_200_restrictions

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.annotation_infos_by_task_id_response_200_item_task_type_settings_resolution_restrictions import (
        AnnotationInfosByTaskIdResponse200ItemTaskTypeSettingsResolutionRestrictions,
    )


T = TypeVar("T", bound="AnnotationInfosByTaskIdResponse200ItemTaskTypeSettings")


@attr.s(auto_attribs=True)
class AnnotationInfosByTaskIdResponse200ItemTaskTypeSettings:
    """
    Attributes:
        allowed_modes (Union[Unset, List[str]]):
        branch_points_allowed (Union[Unset, int]):
        soma_clicking_allowed (Union[Unset, int]):
        volume_interpolation_allowed (Union[Unset, int]):
        merger_mode (Union[Unset, int]):
        resolution_restrictions (Union[Unset,
            AnnotationInfosByTaskIdResponse200ItemTaskTypeSettingsResolutionRestrictions]):
    """

    allowed_modes: Union[Unset, List[str]] = UNSET
    branch_points_allowed: Union[Unset, int] = UNSET
    soma_clicking_allowed: Union[Unset, int] = UNSET
    volume_interpolation_allowed: Union[Unset, int] = UNSET
    merger_mode: Union[Unset, int] = UNSET
    resolution_restrictions: Union[
        Unset,
        "AnnotationInfosByTaskIdResponse200ItemTaskTypeSettingsResolutionRestrictions",
    ] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        allowed_modes: Union[Unset, List[str]] = UNSET
        if not isinstance(self.allowed_modes, Unset):
            allowed_modes = self.allowed_modes

        branch_points_allowed = self.branch_points_allowed
        soma_clicking_allowed = self.soma_clicking_allowed
        volume_interpolation_allowed = self.volume_interpolation_allowed
        merger_mode = self.merger_mode
        resolution_restrictions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.resolution_restrictions, Unset):
            resolution_restrictions = self.resolution_restrictions.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if allowed_modes is not UNSET:
            field_dict["allowedModes"] = allowed_modes
        if branch_points_allowed is not UNSET:
            field_dict["branchPointsAllowed"] = branch_points_allowed
        if soma_clicking_allowed is not UNSET:
            field_dict["somaClickingAllowed"] = soma_clicking_allowed
        if volume_interpolation_allowed is not UNSET:
            field_dict["volumeInterpolationAllowed"] = volume_interpolation_allowed
        if merger_mode is not UNSET:
            field_dict["mergerMode"] = merger_mode
        if resolution_restrictions is not UNSET:
            field_dict["resolutionRestrictions"] = resolution_restrictions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.annotation_infos_by_task_id_response_200_item_task_type_settings_resolution_restrictions import (
            AnnotationInfosByTaskIdResponse200ItemTaskTypeSettingsResolutionRestrictions,
        )

        d = src_dict.copy()
        allowed_modes = cast(List[str], d.pop("allowedModes", UNSET))

        branch_points_allowed = d.pop("branchPointsAllowed", UNSET)

        soma_clicking_allowed = d.pop("somaClickingAllowed", UNSET)

        volume_interpolation_allowed = d.pop("volumeInterpolationAllowed", UNSET)

        merger_mode = d.pop("mergerMode", UNSET)

        _resolution_restrictions = d.pop("resolutionRestrictions", UNSET)
        resolution_restrictions: Union[
            Unset,
            AnnotationInfosByTaskIdResponse200ItemTaskTypeSettingsResolutionRestrictions,
        ]
        if isinstance(_resolution_restrictions, Unset):
            resolution_restrictions = UNSET
        else:
            resolution_restrictions = AnnotationInfosByTaskIdResponse200ItemTaskTypeSettingsResolutionRestrictions.from_dict(
                _resolution_restrictions
            )

        annotation_infos_by_task_id_response_200_item_task_type_settings = cls(
            allowed_modes=allowed_modes,
            branch_points_allowed=branch_points_allowed,
            soma_clicking_allowed=soma_clicking_allowed,
            volume_interpolation_allowed=volume_interpolation_allowed,
            merger_mode=merger_mode,
            resolution_restrictions=resolution_restrictions,
        )

        annotation_infos_by_task_id_response_200_item_task_type_settings.additional_properties = (
            d
        )
        return annotation_infos_by_task_id_response_200_item_task_type_settings

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, List, Optional, Union, cast

import httpx

from ... import errors
from ...client import Client
from ...models.dataset_list_response_200_item import DatasetListResponse200Item
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: Client,
    is_active: Union[Unset, None, bool] = UNSET,
    is_unreported: Union[Unset, None, bool] = UNSET,
    organization_name: Union[Unset, None, str] = UNSET,
    only_my_organization: Union[Unset, None, bool] = UNSET,
    uploader_id: Union[Unset, None, str] = UNSET,
    folder_id: Union[Unset, None, str] = UNSET,
    include_subfolders: Union[Unset, None, bool] = UNSET,
    search_query: Union[Unset, None, str] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    compact: Union[Unset, None, bool] = UNSET,
) -> Dict[str, Any]:
    url = "{}/api/datasets".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    params["isActive"] = is_active

    params["isUnreported"] = is_unreported

    params["organizationName"] = organization_name

    params["onlyMyOrganization"] = only_my_organization

    params["uploaderId"] = uploader_id

    params["folderId"] = folder_id

    params["includeSubfolders"] = include_subfolders

    params["searchQuery"] = search_query

    params["limit"] = limit

    params["compact"] = compact

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "follow_redirects": client.follow_redirects,
        "params": params,
    }


def _parse_response(
    *, client: Client, response: httpx.Response
) -> Optional[Union[Any, List["DatasetListResponse200Item"]]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = DatasetListResponse200Item.from_dict(
                response_200_item_data
            )

            response_200.append(response_200_item)

        return response_200
    if response.status_code == HTTPStatus.BAD_REQUEST:
        response_400 = cast(Any, None)
        return response_400
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Client, response: httpx.Response
) -> Response[Union[Any, List["DatasetListResponse200Item"]]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Client,
    is_active: Union[Unset, None, bool] = UNSET,
    is_unreported: Union[Unset, None, bool] = UNSET,
    organization_name: Union[Unset, None, str] = UNSET,
    only_my_organization: Union[Unset, None, bool] = UNSET,
    uploader_id: Union[Unset, None, str] = UNSET,
    folder_id: Union[Unset, None, str] = UNSET,
    include_subfolders: Union[Unset, None, bool] = UNSET,
    search_query: Union[Unset, None, str] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    compact: Union[Unset, None, bool] = UNSET,
) -> Response[Union[Any, List["DatasetListResponse200Item"]]]:
    """List all accessible datasets.

    Args:
        is_active (Union[Unset, None, bool]):
        is_unreported (Union[Unset, None, bool]):
        organization_name (Union[Unset, None, str]):
        only_my_organization (Union[Unset, None, bool]):
        uploader_id (Union[Unset, None, str]):
        folder_id (Union[Unset, None, str]):
        include_subfolders (Union[Unset, None, bool]):
        search_query (Union[Unset, None, str]):
        limit (Union[Unset, None, int]):
        compact (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, List['DatasetListResponse200Item']]]
    """

    kwargs = _get_kwargs(
        client=client,
        is_active=is_active,
        is_unreported=is_unreported,
        organization_name=organization_name,
        only_my_organization=only_my_organization,
        uploader_id=uploader_id,
        folder_id=folder_id,
        include_subfolders=include_subfolders,
        search_query=search_query,
        limit=limit,
        compact=compact,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: Client,
    is_active: Union[Unset, None, bool] = UNSET,
    is_unreported: Union[Unset, None, bool] = UNSET,
    organization_name: Union[Unset, None, str] = UNSET,
    only_my_organization: Union[Unset, None, bool] = UNSET,
    uploader_id: Union[Unset, None, str] = UNSET,
    folder_id: Union[Unset, None, str] = UNSET,
    include_subfolders: Union[Unset, None, bool] = UNSET,
    search_query: Union[Unset, None, str] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    compact: Union[Unset, None, bool] = UNSET,
) -> Optional[Union[Any, List["DatasetListResponse200Item"]]]:
    """List all accessible datasets.

    Args:
        is_active (Union[Unset, None, bool]):
        is_unreported (Union[Unset, None, bool]):
        organization_name (Union[Unset, None, str]):
        only_my_organization (Union[Unset, None, bool]):
        uploader_id (Union[Unset, None, str]):
        folder_id (Union[Unset, None, str]):
        include_subfolders (Union[Unset, None, bool]):
        search_query (Union[Unset, None, str]):
        limit (Union[Unset, None, int]):
        compact (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, List['DatasetListResponse200Item']]
    """

    return sync_detailed(
        client=client,
        is_active=is_active,
        is_unreported=is_unreported,
        organization_name=organization_name,
        only_my_organization=only_my_organization,
        uploader_id=uploader_id,
        folder_id=folder_id,
        include_subfolders=include_subfolders,
        search_query=search_query,
        limit=limit,
        compact=compact,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    is_active: Union[Unset, None, bool] = UNSET,
    is_unreported: Union[Unset, None, bool] = UNSET,
    organization_name: Union[Unset, None, str] = UNSET,
    only_my_organization: Union[Unset, None, bool] = UNSET,
    uploader_id: Union[Unset, None, str] = UNSET,
    folder_id: Union[Unset, None, str] = UNSET,
    include_subfolders: Union[Unset, None, bool] = UNSET,
    search_query: Union[Unset, None, str] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    compact: Union[Unset, None, bool] = UNSET,
) -> Response[Union[Any, List["DatasetListResponse200Item"]]]:
    """List all accessible datasets.

    Args:
        is_active (Union[Unset, None, bool]):
        is_unreported (Union[Unset, None, bool]):
        organization_name (Union[Unset, None, str]):
        only_my_organization (Union[Unset, None, bool]):
        uploader_id (Union[Unset, None, str]):
        folder_id (Union[Unset, None, str]):
        include_subfolders (Union[Unset, None, bool]):
        search_query (Union[Unset, None, str]):
        limit (Union[Unset, None, int]):
        compact (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, List['DatasetListResponse200Item']]]
    """

    kwargs = _get_kwargs(
        client=client,
        is_active=is_active,
        is_unreported=is_unreported,
        organization_name=organization_name,
        only_my_organization=only_my_organization,
        uploader_id=uploader_id,
        folder_id=folder_id,
        include_subfolders=include_subfolders,
        search_query=search_query,
        limit=limit,
        compact=compact,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: Client,
    is_active: Union[Unset, None, bool] = UNSET,
    is_unreported: Union[Unset, None, bool] = UNSET,
    organization_name: Union[Unset, None, str] = UNSET,
    only_my_organization: Union[Unset, None, bool] = UNSET,
    uploader_id: Union[Unset, None, str] = UNSET,
    folder_id: Union[Unset, None, str] = UNSET,
    include_subfolders: Union[Unset, None, bool] = UNSET,
    search_query: Union[Unset, None, str] = UNSET,
    limit: Union[Unset, None, int] = UNSET,
    compact: Union[Unset, None, bool] = UNSET,
) -> Optional[Union[Any, List["DatasetListResponse200Item"]]]:
    """List all accessible datasets.

    Args:
        is_active (Union[Unset, None, bool]):
        is_unreported (Union[Unset, None, bool]):
        organization_name (Union[Unset, None, str]):
        only_my_organization (Union[Unset, None, bool]):
        uploader_id (Union[Unset, None, str]):
        folder_id (Union[Unset, None, str]):
        include_subfolders (Union[Unset, None, bool]):
        search_query (Union[Unset, None, str]):
        limit (Union[Unset, None, int]):
        compact (Union[Unset, None, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, List['DatasetListResponse200Item']]
    """

    return (
        await asyncio_detailed(
            client=client,
            is_active=is_active,
            is_unreported=is_unreported,
            organization_name=organization_name,
            only_my_organization=only_my_organization,
            uploader_id=uploader_id,
            folder_id=folder_id,
            include_subfolders=include_subfolders,
            search_query=search_query,
            limit=limit,
            compact=compact,
        )
    ).parsed



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.current_user_info_response_200_experiences import (
        CurrentUserInfoResponse200Experiences,
    )
    from ..models.current_user_info_response_200_novel_user_experience_infos import (
        CurrentUserInfoResponse200NovelUserExperienceInfos,
    )
    from ..models.current_user_info_response_200_teams_item import (
        CurrentUserInfoResponse200TeamsItem,
    )


T = TypeVar("T", bound="CurrentUserInfoResponse200")


@attr.s(auto_attribs=True)
class CurrentUserInfoResponse200:
    """
    Attributes:
        id (str):
        email (str):
        first_name (str):
        last_name (str):
        is_admin (int):
        is_dataset_manager (int):
        is_active (int):
        experiences (CurrentUserInfoResponse200Experiences):
        last_activity (int):
        organization (str):
        created (int):
        is_organization_owner (Union[Unset, int]):
        teams (Union[Unset, List['CurrentUserInfoResponse200TeamsItem']]):
        is_anonymous (Union[Unset, int]):
        is_editable (Union[Unset, int]):
        novel_user_experience_infos (Union[Unset, CurrentUserInfoResponse200NovelUserExperienceInfos]):
        selected_theme (Union[Unset, str]):
        last_task_type_id (Union[Unset, str]):
        is_super_user (Union[Unset, int]):
        is_email_verified (Union[Unset, int]):
    """

    id: str
    email: str
    first_name: str
    last_name: str
    is_admin: int
    is_dataset_manager: int
    is_active: int
    experiences: "CurrentUserInfoResponse200Experiences"
    last_activity: int
    organization: str
    created: int
    is_organization_owner: Union[Unset, int] = UNSET
    teams: Union[Unset, List["CurrentUserInfoResponse200TeamsItem"]] = UNSET
    is_anonymous: Union[Unset, int] = UNSET
    is_editable: Union[Unset, int] = UNSET
    novel_user_experience_infos: Union[
        Unset, "CurrentUserInfoResponse200NovelUserExperienceInfos"
    ] = UNSET
    selected_theme: Union[Unset, str] = UNSET
    last_task_type_id: Union[Unset, str] = UNSET
    is_super_user: Union[Unset, int] = UNSET
    is_email_verified: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        email = self.email
        first_name = self.first_name
        last_name = self.last_name
        is_admin = self.is_admin
        is_dataset_manager = self.is_dataset_manager
        is_active = self.is_active
        experiences = self.experiences.to_dict()

        last_activity = self.last_activity
        organization = self.organization
        created = self.created
        is_organization_owner = self.is_organization_owner
        teams: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.teams, Unset):
            teams = []
            for teams_item_data in self.teams:
                teams_item = teams_item_data.to_dict()

                teams.append(teams_item)

        is_anonymous = self.is_anonymous
        is_editable = self.is_editable
        novel_user_experience_infos: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.novel_user_experience_infos, Unset):
            novel_user_experience_infos = self.novel_user_experience_infos.to_dict()

        selected_theme = self.selected_theme
        last_task_type_id = self.last_task_type_id
        is_super_user = self.is_super_user
        is_email_verified = self.is_email_verified

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "email": email,
                "firstName": first_name,
                "lastName": last_name,
                "isAdmin": is_admin,
                "isDatasetManager": is_dataset_manager,
                "isActive": is_active,
                "experiences": experiences,
                "lastActivity": last_activity,
                "organization": organization,
                "created": created,
            }
        )
        if is_organization_owner is not UNSET:
            field_dict["isOrganizationOwner"] = is_organization_owner
        if teams is not UNSET:
            field_dict["teams"] = teams
        if is_anonymous is not UNSET:
            field_dict["isAnonymous"] = is_anonymous
        if is_editable is not UNSET:
            field_dict["isEditable"] = is_editable
        if novel_user_experience_infos is not UNSET:
            field_dict["novelUserExperienceInfos"] = novel_user_experience_infos
        if selected_theme is not UNSET:
            field_dict["selectedTheme"] = selected_theme
        if last_task_type_id is not UNSET:
            field_dict["lastTaskTypeId"] = last_task_type_id
        if is_super_user is not UNSET:
            field_dict["isSuperUser"] = is_super_user
        if is_email_verified is not UNSET:
            field_dict["isEmailVerified"] = is_email_verified

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.current_user_info_response_200_experiences import (
            CurrentUserInfoResponse200Experiences,
        )
        from ..models.current_user_info_response_200_novel_user_experience_infos import (
            CurrentUserInfoResponse200NovelUserExperienceInfos,
        )
        from ..models.current_user_info_response_200_teams_item import (
            CurrentUserInfoResponse200TeamsItem,
        )

        d = src_dict.copy()
        id = d.pop("id")

        email = d.pop("email")

        first_name = d.pop("firstName")

        last_name = d.pop("lastName")

        is_admin = d.pop("isAdmin")

        is_dataset_manager = d.pop("isDatasetManager")

        is_active = d.pop("isActive")

        experiences = CurrentUserInfoResponse200Experiences.from_dict(
            d.pop("experiences")
        )

        last_activity = d.pop("lastActivity")

        organization = d.pop("organization")

        created = d.pop("created")

        is_organization_owner = d.pop("isOrganizationOwner", UNSET)

        teams = []
        _teams = d.pop("teams", UNSET)
        for teams_item_data in _teams or []:
            teams_item = CurrentUserInfoResponse200TeamsItem.from_dict(teams_item_data)

            teams.append(teams_item)

        is_anonymous = d.pop("isAnonymous", UNSET)

        is_editable = d.pop("isEditable", UNSET)

        _novel_user_experience_infos = d.pop("novelUserExperienceInfos", UNSET)
        novel_user_experience_infos: Union[
            Unset, CurrentUserInfoResponse200NovelUserExperienceInfos
        ]
        if isinstance(_novel_user_experience_infos, Unset):
            novel_user_experience_infos = UNSET
        else:
            novel_user_experience_infos = (
                CurrentUserInfoResponse200NovelUserExperienceInfos.from_dict(
                    _novel_user_experience_infos
                )
            )

        selected_theme = d.pop("selectedTheme", UNSET)

        last_task_type_id = d.pop("lastTaskTypeId", UNSET)

        is_super_user = d.pop("isSuperUser", UNSET)

        is_email_verified = d.pop("isEmailVerified", UNSET)

        current_user_info_response_200 = cls(
            id=id,
            email=email,
            first_name=first_name,
            last_name=last_name,
            is_admin=is_admin,
            is_dataset_manager=is_dataset_manager,
            is_active=is_active,
            experiences=experiences,
            last_activity=last_activity,
            organization=organization,
            created=created,
            is_organization_owner=is_organization_owner,
            teams=teams,
            is_anonymous=is_anonymous,
            is_editable=is_editable,
            novel_user_experience_infos=novel_user_experience_infos,
            selected_theme=selected_theme,
            last_task_type_id=last_task_type_id,
            is_super_user=is_super_user,
            is_email_verified=is_email_verified,
        )

        current_user_info_response_200.additional_properties = d
        return current_user_info_response_200

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Tuple, Union, cast

import attr

from ..client._generated.api.default import (
    current_user_info,
    team_list,
    user_info_by_id,
    user_list,
    user_logged_time,
)
from ..client._generated.types import Unset

if TYPE_CHECKING:
    from ..client._generated.models.current_user_info_response_200 import (
        CurrentUserInfoResponse200,
    )
    from ..client._generated.models.user_list_response_200_item import (
        UserListResponse200Item,
    )
    from ..client._generated.models.user_info_by_id_response_200 import (
        UserInfoByIdResponse200,
    )

from ..client.context import _get_generated_client


@attr.frozen
class User:
    """Represents a user of a webknossos instance.
    You can get users via `get_current_user` and `get_all_managed_users`."""

    user_id: str
    email: str
    organization_id: str
    first_name: str
    last_name: str
    created: int
    last_activity: int
    teams: Tuple["Team", ...]
    experiences: Dict[str, int]
    is_active: bool
    is_admin: bool
    is_dataset_manager: bool

    def get_logged_times(self) -> List["LoggedTime"]:
        """Get the logged times of this user.
        Returns a list of `LoggedTime` objects where one represents one month."""
        client = _get_generated_client(enforce_auth=True)
        response = user_logged_time.sync(id=self.user_id, client=client)
        assert response is not None, f"Could not fetch logged time of {self}"
        return [
            LoggedTime(
                duration_in_seconds=i.duration_in_seconds,
                month=i.payment_interval.month,
                year=i.payment_interval.year,
            )
            for i in response.logged_time
        ]

    @classmethod
    def _from_generated_response(
        cls,
        response: Union[
            "UserListResponse200Item",
            "CurrentUserInfoResponse200",
            "UserInfoByIdResponse200",
        ],
    ) -> "User":
        if isinstance(response.teams, Unset):
            response.teams = cast(List[Any], [])
        return cls(
            user_id=response.id,
            email=response.email,
            organization_id=response.organization,
            first_name=response.first_name,
            last_name=response.last_name,
            created=response.created,
            last_activity=response.last_activity,
            teams=tuple(
                Team(id=team.id, name=team.name, organization_id=response.organization)
                for team in response.teams
            ),
            experiences=response.experiences.additional_properties,
            is_active=bool(response.is_active),
            is_admin=bool(response.is_admin),
            is_dataset_manager=bool(response.is_dataset_manager),
        )

    @classmethod
    def get_by_id(cls, id: str) -> "User":  # pylint: disable=redefined-builtin
        """Returns the user specified by the passed id if your token authorizes you to see them."""
        client = _get_generated_client(enforce_auth=True)
        response = user_info_by_id.sync(id, client=client)
        assert response is not None, "Could not fetch user by id."
        return cls._from_generated_response(response)

    @classmethod
    def get_current_user(cls) -> "User":
        """Returns the current user from the authentication context."""
        client = _get_generated_client(enforce_auth=True)
        response = current_user_info.sync(client=client)
        assert response is not None, "Could not fetch current user."
        return cls._from_generated_response(response)

    @classmethod
    def get_all_managed_users(cls) -> List["User"]:
        """Returns all users of whom the current user is admin or team-manager."""
        client = _get_generated_client(enforce_auth=True)
        response = user_list.sync(client=client)
        assert response is not None, "Could not fetch managed users."
        return [cls._from_generated_response(i) for i in response]


@attr.frozen
class Team:
    id: str
    name: str
    organization_id: str

    @classmethod
    def get_by_name(cls, name: str) -> "Team":
        """Returns the Team specified by the passed name if your token authorizes you to see it."""
        client = _get_generated_client(enforce_auth=True)
        response = team_list.sync(client=client)
        assert response is not None, "Could not fetch teams."
        for team in response:
            if team.name == name:
                return cls(team.id, team.name, team.organization)
        raise KeyError(f"Could not find team {name}.")


@attr.frozen
class LoggedTime:
    duration_in_seconds: int
    year: int
    month: int



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

T = TypeVar(
    "T", bound="DatasetInfoResponse200DataSourceDataLayersItemDefaultViewConfiguration"
)


@attr.s(auto_attribs=True)
class DatasetInfoResponse200DataSourceDataLayersItemDefaultViewConfiguration:
    """
    Attributes:
        color (Union[Unset, List[int]]):
    """

    color: Union[Unset, List[int]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        color: Union[Unset, List[int]] = UNSET
        if not isinstance(self.color, Unset):
            color = self.color

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if color is not UNSET:
            field_dict["color"] = color

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        color = cast(List[int], d.pop("color", UNSET))

        dataset_info_response_200_data_source_data_layers_item_default_view_configuration = cls(
            color=color,
        )

        dataset_info_response_200_data_source_data_layers_item_default_view_configuration.additional_properties = (
            d
        )
        return dataset_info_response_200_data_source_data_layers_item_default_view_configuration

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.project_info_by_name_response_200_owner_teams_item import (
        ProjectInfoByNameResponse200OwnerTeamsItem,
    )


T = TypeVar("T", bound="ProjectInfoByNameResponse200Owner")


@attr.s(auto_attribs=True)
class ProjectInfoByNameResponse200Owner:
    """
    Attributes:
        id (str):
        email (str):
        first_name (str):
        last_name (str):
        is_admin (int):
        is_dataset_manager (int):
        is_anonymous (Union[Unset, int]):
        teams (Union[Unset, List['ProjectInfoByNameResponse200OwnerTeamsItem']]):
    """

    id: str
    email: str
    first_name: str
    last_name: str
    is_admin: int
    is_dataset_manager: int
    is_anonymous: Union[Unset, int] = UNSET
    teams: Union[Unset, List["ProjectInfoByNameResponse200OwnerTeamsItem"]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        email = self.email
        first_name = self.first_name
        last_name = self.last_name
        is_admin = self.is_admin
        is_dataset_manager = self.is_dataset_manager
        is_anonymous = self.is_anonymous
        teams: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.teams, Unset):
            teams = []
            for teams_item_data in self.teams:
                teams_item = teams_item_data.to_dict()

                teams.append(teams_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "email": email,
                "firstName": first_name,
                "lastName": last_name,
                "isAdmin": is_admin,
                "isDatasetManager": is_dataset_manager,
            }
        )
        if is_anonymous is not UNSET:
            field_dict["isAnonymous"] = is_anonymous
        if teams is not UNSET:
            field_dict["teams"] = teams

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.project_info_by_name_response_200_owner_teams_item import (
            ProjectInfoByNameResponse200OwnerTeamsItem,
        )

        d = src_dict.copy()
        id = d.pop("id")

        email = d.pop("email")

        first_name = d.pop("firstName")

        last_name = d.pop("lastName")

        is_admin = d.pop("isAdmin")

        is_dataset_manager = d.pop("isDatasetManager")

        is_anonymous = d.pop("isAnonymous", UNSET)

        teams = []
        _teams = d.pop("teams", UNSET)
        for teams_item_data in _teams or []:
            teams_item = ProjectInfoByNameResponse200OwnerTeamsItem.from_dict(
                teams_item_data
            )

            teams.append(teams_item)

        project_info_by_name_response_200_owner = cls(
            id=id,
            email=email,
            first_name=first_name,
            last_name=last_name,
            is_admin=is_admin,
            is_dataset_manager=is_dataset_manager,
            is_anonymous=is_anonymous,
            teams=teams,
        )

        project_info_by_name_response_200_owner.additional_properties = d
        return project_info_by_name_response_200_owner

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



""" Contains all the data models used in inputs/outputs """

from .action import Action
from .action_annotation_layer_parameters import ActionAnnotationLayerParameters
from .action_annotation_private_link_params import ActionAnnotationPrivateLinkParams
from .action_any_content import ActionAnyContent
from .action_cancel_upload_information import ActionCancelUploadInformation
from .action_dataset_update_parameters import ActionDatasetUpdateParameters
from .action_js_value import ActionJsValue
from .action_list_object_id import ActionListObjectId
from .action_multipart_form_data_temporary_file import (
    ActionMultipartFormDataTemporaryFile,
)
from .action_reserve_upload_information import ActionReserveUploadInformation
from .action_upload_information import ActionUploadInformation
from .annotation_info_response_200 import AnnotationInfoResponse200
from .annotation_info_response_200_annotation_layers_item import (
    AnnotationInfoResponse200AnnotationLayersItem,
)
from .annotation_info_response_200_data_store import AnnotationInfoResponse200DataStore
from .annotation_info_response_200_owner import AnnotationInfoResponse200Owner
from .annotation_info_response_200_owner_teams_item import (
    AnnotationInfoResponse200OwnerTeamsItem,
)
from .annotation_info_response_200_restrictions import (
    AnnotationInfoResponse200Restrictions,
)
from .annotation_info_response_200_settings import AnnotationInfoResponse200Settings
from .annotation_info_response_200_settings_resolution_restrictions import (
    AnnotationInfoResponse200SettingsResolutionRestrictions,
)
from .annotation_info_response_200_stats import AnnotationInfoResponse200Stats
from .annotation_info_response_200_task import AnnotationInfoResponse200Task
from .annotation_info_response_200_task_needed_experience import (
    AnnotationInfoResponse200TaskNeededExperience,
)
from .annotation_info_response_200_task_status import (
    AnnotationInfoResponse200TaskStatus,
)
from .annotation_info_response_200_task_type import AnnotationInfoResponse200TaskType
from .annotation_info_response_200_task_type_settings import (
    AnnotationInfoResponse200TaskTypeSettings,
)
from .annotation_info_response_200_task_type_settings_resolution_restrictions import (
    AnnotationInfoResponse200TaskTypeSettingsResolutionRestrictions,
)
from .annotation_info_response_200_tracing_store import (
    AnnotationInfoResponse200TracingStore,
)
from .annotation_info_response_200_user import AnnotationInfoResponse200User
from .annotation_info_response_200_user_teams_item import (
    AnnotationInfoResponse200UserTeamsItem,
)
from .annotation_infos_by_task_id_response_200_item import (
    AnnotationInfosByTaskIdResponse200Item,
)
from .annotation_infos_by_task_id_response_200_item_annotation_layers_item import (
    AnnotationInfosByTaskIdResponse200ItemAnnotationLayersItem,
)
from .annotation_infos_by_task_id_response_200_item_data_store import (
    AnnotationInfosByTaskIdResponse200ItemDataStore,
)
from .annotation_infos_by_task_id_response_200_item_owner import (
    AnnotationInfosByTaskIdResponse200ItemOwner,
)
from .annotation_infos_by_task_id_response_200_item_owner_teams_item import (
    AnnotationInfosByTaskIdResponse200ItemOwnerTeamsItem,
)
from .annotation_infos_by_task_id_response_200_item_restrictions import (
    AnnotationInfosByTaskIdResponse200ItemRestrictions,
)
from .annotation_infos_by_task_id_response_200_item_settings import (
    AnnotationInfosByTaskIdResponse200ItemSettings,
)
from .annotation_infos_by_task_id_response_200_item_settings_resolution_restrictions import (
    AnnotationInfosByTaskIdResponse200ItemSettingsResolutionRestrictions,
)
from .annotation_infos_by_task_id_response_200_item_stats import (
    AnnotationInfosByTaskIdResponse200ItemStats,
)
from .annotation_infos_by_task_id_response_200_item_task import (
    AnnotationInfosByTaskIdResponse200ItemTask,
)
from .annotation_infos_by_task_id_response_200_item_task_needed_experience import (
    AnnotationInfosByTaskIdResponse200ItemTaskNeededExperience,
)
from .annotation_infos_by_task_id_response_200_item_task_status import (
    AnnotationInfosByTaskIdResponse200ItemTaskStatus,
)
from .annotation_infos_by_task_id_response_200_item_task_type import (
    AnnotationInfosByTaskIdResponse200ItemTaskType,
)
from .annotation_infos_by_task_id_response_200_item_task_type_settings import (
    AnnotationInfosByTaskIdResponse200ItemTaskTypeSettings,
)
from .annotation_infos_by_task_id_response_200_item_task_type_settings_resolution_restrictions import (
    AnnotationInfosByTaskIdResponse200ItemTaskTypeSettingsResolutionRestrictions,
)
from .annotation_infos_by_task_id_response_200_item_tracing_store import (
    AnnotationInfosByTaskIdResponse200ItemTracingStore,
)
from .annotation_infos_by_task_id_response_200_item_user import (
    AnnotationInfosByTaskIdResponse200ItemUser,
)
from .annotation_infos_by_task_id_response_200_item_user_teams_item import (
    AnnotationInfosByTaskIdResponse200ItemUserTeamsItem,
)
from .annotation_private_link_params import AnnotationPrivateLinkParams
from .build_info_response_200 import BuildInfoResponse200
from .build_info_response_200_webknossos import BuildInfoResponse200Webknossos
from .build_info_response_200_webknossos_wrap import BuildInfoResponse200WebknossosWrap
from .cancel_upload_information import CancelUploadInformation
from .create_private_link_json_body import CreatePrivateLinkJsonBody
from .create_project_json_body import CreateProjectJsonBody
from .current_user_info_response_200 import CurrentUserInfoResponse200
from .current_user_info_response_200_experiences import (
    CurrentUserInfoResponse200Experiences,
)
from .current_user_info_response_200_novel_user_experience_infos import (
    CurrentUserInfoResponse200NovelUserExperienceInfos,
)
from .current_user_info_response_200_teams_item import (
    CurrentUserInfoResponse200TeamsItem,
)
from .dataset_cancel_upload_json_body import DatasetCancelUploadJsonBody
from .dataset_finish_upload_json_body import DatasetFinishUploadJsonBody
from .dataset_info_response_200 import DatasetInfoResponse200
from .dataset_info_response_200_allowed_teams_cumulative_item import (
    DatasetInfoResponse200AllowedTeamsCumulativeItem,
)
from .dataset_info_response_200_allowed_teams_item import (
    DatasetInfoResponse200AllowedTeamsItem,
)
from .dataset_info_response_200_data_source import DatasetInfoResponse200DataSource
from .dataset_info_response_200_data_source_data_layers_item import (
    DatasetInfoResponse200DataSourceDataLayersItem,
)
from .dataset_info_response_200_data_source_data_layers_item_bounding_box import (
    DatasetInfoResponse200DataSourceDataLayersItemBoundingBox,
)
from .dataset_info_response_200_data_source_data_layers_item_default_view_configuration import (
    DatasetInfoResponse200DataSourceDataLayersItemDefaultViewConfiguration,
)
from .dataset_info_response_200_data_source_id import DatasetInfoResponse200DataSourceId
from .dataset_info_response_200_data_store import DatasetInfoResponse200DataStore
from .dataset_list_response_200_item import DatasetListResponse200Item
from .dataset_list_response_200_item_allowed_teams_cumulative_item import (
    DatasetListResponse200ItemAllowedTeamsCumulativeItem,
)
from .dataset_list_response_200_item_allowed_teams_item import (
    DatasetListResponse200ItemAllowedTeamsItem,
)
from .dataset_list_response_200_item_data_source import (
    DatasetListResponse200ItemDataSource,
)
from .dataset_list_response_200_item_data_source_id import (
    DatasetListResponse200ItemDataSourceId,
)
from .dataset_list_response_200_item_data_store import (
    DatasetListResponse200ItemDataStore,
)
from .dataset_reserve_upload_json_body import DatasetReserveUploadJsonBody
from .dataset_sharing_token_response_200 import DatasetSharingTokenResponse200
from .dataset_update_json_body import DatasetUpdateJsonBody
from .dataset_update_partial_json_body import DatasetUpdatePartialJsonBody
from .datastore_list_response_200_item import DatastoreListResponse200Item
from .folder_tree_response_200_item import FolderTreeResponse200Item
from .generate_token_for_data_store_response_200 import (
    GenerateTokenForDataStoreResponse200,
)
from .instant import Instant
from .js_object import JsObject
from .js_object_underlying import JsObjectUnderlying
from .js_object_value import JsObjectValue
from .js_value import JsValue
from .linked_layer_identifier import LinkedLayerIdentifier
from .project_info_by_id_response_200 import ProjectInfoByIdResponse200
from .project_info_by_id_response_200_owner import ProjectInfoByIdResponse200Owner
from .project_info_by_id_response_200_owner_teams_item import (
    ProjectInfoByIdResponse200OwnerTeamsItem,
)
from .project_info_by_name_response_200 import ProjectInfoByNameResponse200
from .project_info_by_name_response_200_owner import ProjectInfoByNameResponse200Owner
from .project_info_by_name_response_200_owner_teams_item import (
    ProjectInfoByNameResponse200OwnerTeamsItem,
)
from .reserve_upload_information import ReserveUploadInformation
from .short_link_by_key_response_200 import ShortLinkByKeyResponse200
from .task_create_from_files_json_body import TaskCreateFromFilesJsonBody
from .task_info_response_200 import TaskInfoResponse200
from .task_info_response_200_needed_experience import (
    TaskInfoResponse200NeededExperience,
)
from .task_info_response_200_status import TaskInfoResponse200Status
from .task_info_response_200_type import TaskInfoResponse200Type
from .task_info_response_200_type_settings import TaskInfoResponse200TypeSettings
from .task_info_response_200_type_settings_resolution_restrictions import (
    TaskInfoResponse200TypeSettingsResolutionRestrictions,
)
from .task_infos_by_project_id_response_200_item import (
    TaskInfosByProjectIdResponse200Item,
)
from .task_infos_by_project_id_response_200_item_needed_experience import (
    TaskInfosByProjectIdResponse200ItemNeededExperience,
)
from .task_infos_by_project_id_response_200_item_status import (
    TaskInfosByProjectIdResponse200ItemStatus,
)
from .task_infos_by_project_id_response_200_item_type import (
    TaskInfosByProjectIdResponse200ItemType,
)
from .task_infos_by_project_id_response_200_item_type_settings import (
    TaskInfosByProjectIdResponse200ItemTypeSettings,
)
from .task_infos_by_project_id_response_200_item_type_settings_resolution_restrictions import (
    TaskInfosByProjectIdResponse200ItemTypeSettingsResolutionRestrictions,
)
from .team_list_response_200_item import TeamListResponse200Item
from .tuple_2 import Tuple2
from .tuple_2_string_js_value import Tuple2StringJsValue
from .tuple_21 import Tuple21
from .tuple_22 import Tuple22
from .update_private_link_json_body import UpdatePrivateLinkJsonBody
from .upload_information import UploadInformation
from .upload_information_needs_conversion import UploadInformationNeedsConversion
from .user_info_by_id_response_200 import UserInfoByIdResponse200
from .user_info_by_id_response_200_experiences import UserInfoByIdResponse200Experiences
from .user_info_by_id_response_200_novel_user_experience_infos import (
    UserInfoByIdResponse200NovelUserExperienceInfos,
)
from .user_info_by_id_response_200_teams_item import UserInfoByIdResponse200TeamsItem
from .user_list_response_200_item import UserListResponse200Item
from .user_list_response_200_item_experiences import UserListResponse200ItemExperiences
from .user_list_response_200_item_novel_user_experience_infos import (
    UserListResponse200ItemNovelUserExperienceInfos,
)
from .user_list_response_200_item_teams_item import UserListResponse200ItemTeamsItem
from .user_logged_time_response_200 import UserLoggedTimeResponse200
from .user_logged_time_response_200_logged_time_item import (
    UserLoggedTimeResponse200LoggedTimeItem,
)
from .user_logged_time_response_200_logged_time_item_payment_interval import (
    UserLoggedTimeResponse200LoggedTimeItemPaymentInterval,
)

__all__ = (
    "Action",
    "ActionAnnotationLayerParameters",
    "ActionAnnotationPrivateLinkParams",
    "ActionAnyContent",
    "ActionCancelUploadInformation",
    "ActionDatasetUpdateParameters",
    "ActionJsValue",
    "ActionListObjectId",
    "ActionMultipartFormDataTemporaryFile",
    "ActionReserveUploadInformation",
    "ActionUploadInformation",
    "AnnotationInfoResponse200",
    "AnnotationInfoResponse200AnnotationLayersItem",
    "AnnotationInfoResponse200DataStore",
    "AnnotationInfoResponse200Owner",
    "AnnotationInfoResponse200OwnerTeamsItem",
    "AnnotationInfoResponse200Restrictions",
    "AnnotationInfoResponse200Settings",
    "AnnotationInfoResponse200SettingsResolutionRestrictions",
    "AnnotationInfoResponse200Stats",
    "AnnotationInfoResponse200Task",
    "AnnotationInfoResponse200TaskNeededExperience",
    "AnnotationInfoResponse200TaskStatus",
    "AnnotationInfoResponse200TaskType",
    "AnnotationInfoResponse200TaskTypeSettings",
    "AnnotationInfoResponse200TaskTypeSettingsResolutionRestrictions",
    "AnnotationInfoResponse200TracingStore",
    "AnnotationInfoResponse200User",
    "AnnotationInfoResponse200UserTeamsItem",
    "AnnotationInfosByTaskIdResponse200Item",
    "AnnotationInfosByTaskIdResponse200ItemAnnotationLayersItem",
    "AnnotationInfosByTaskIdResponse200ItemDataStore",
    "AnnotationInfosByTaskIdResponse200ItemOwner",
    "AnnotationInfosByTaskIdResponse200ItemOwnerTeamsItem",
    "AnnotationInfosByTaskIdResponse200ItemRestrictions",
    "AnnotationInfosByTaskIdResponse200ItemSettings",
    "AnnotationInfosByTaskIdResponse200ItemSettingsResolutionRestrictions",
    "AnnotationInfosByTaskIdResponse200ItemStats",
    "AnnotationInfosByTaskIdResponse200ItemTask",
    "AnnotationInfosByTaskIdResponse200ItemTaskNeededExperience",
    "AnnotationInfosByTaskIdResponse200ItemTaskStatus",
    "AnnotationInfosByTaskIdResponse200ItemTaskType",
    "AnnotationInfosByTaskIdResponse200ItemTaskTypeSettings",
    "AnnotationInfosByTaskIdResponse200ItemTaskTypeSettingsResolutionRestrictions",
    "AnnotationInfosByTaskIdResponse200ItemTracingStore",
    "AnnotationInfosByTaskIdResponse200ItemUser",
    "AnnotationInfosByTaskIdResponse200ItemUserTeamsItem",
    "AnnotationPrivateLinkParams",
    "BuildInfoResponse200",
    "BuildInfoResponse200Webknossos",
    "BuildInfoResponse200WebknossosWrap",
    "CancelUploadInformation",
    "CreatePrivateLinkJsonBody",
    "CreateProjectJsonBody",
    "CurrentUserInfoResponse200",
    "CurrentUserInfoResponse200Experiences",
    "CurrentUserInfoResponse200NovelUserExperienceInfos",
    "CurrentUserInfoResponse200TeamsItem",
    "DatasetCancelUploadJsonBody",
    "DatasetFinishUploadJsonBody",
    "DatasetInfoResponse200",
    "DatasetInfoResponse200AllowedTeamsCumulativeItem",
    "DatasetInfoResponse200AllowedTeamsItem",
    "DatasetInfoResponse200DataSource",
    "DatasetInfoResponse200DataSourceDataLayersItem",
    "DatasetInfoResponse200DataSourceDataLayersItemBoundingBox",
    "DatasetInfoResponse200DataSourceDataLayersItemDefaultViewConfiguration",
    "DatasetInfoResponse200DataSourceId",
    "DatasetInfoResponse200DataStore",
    "DatasetListResponse200Item",
    "DatasetListResponse200ItemAllowedTeamsCumulativeItem",
    "DatasetListResponse200ItemAllowedTeamsItem",
    "DatasetListResponse200ItemDataSource",
    "DatasetListResponse200ItemDataSourceId",
    "DatasetListResponse200ItemDataStore",
    "DatasetReserveUploadJsonBody",
    "DatasetSharingTokenResponse200",
    "DatasetUpdateJsonBody",
    "DatasetUpdatePartialJsonBody",
    "DatastoreListResponse200Item",
    "FolderTreeResponse200Item",
    "GenerateTokenForDataStoreResponse200",
    "Instant",
    "JsObject",
    "JsObjectUnderlying",
    "JsObjectValue",
    "JsValue",
    "LinkedLayerIdentifier",
    "ProjectInfoByIdResponse200",
    "ProjectInfoByIdResponse200Owner",
    "ProjectInfoByIdResponse200OwnerTeamsItem",
    "ProjectInfoByNameResponse200",
    "ProjectInfoByNameResponse200Owner",
    "ProjectInfoByNameResponse200OwnerTeamsItem",
    "ReserveUploadInformation",
    "ShortLinkByKeyResponse200",
    "TaskCreateFromFilesJsonBody",
    "TaskInfoResponse200",
    "TaskInfoResponse200NeededExperience",
    "TaskInfoResponse200Status",
    "TaskInfoResponse200Type",
    "TaskInfoResponse200TypeSettings",
    "TaskInfoResponse200TypeSettingsResolutionRestrictions",
    "TaskInfosByProjectIdResponse200Item",
    "TaskInfosByProjectIdResponse200ItemNeededExperience",
    "TaskInfosByProjectIdResponse200ItemStatus",
    "TaskInfosByProjectIdResponse200ItemType",
    "TaskInfosByProjectIdResponse200ItemTypeSettings",
    "TaskInfosByProjectIdResponse200ItemTypeSettingsResolutionRestrictions",
    "TeamListResponse200Item",
    "Tuple2",
    "Tuple21",
    "Tuple22",
    "Tuple2StringJsValue",
    "UpdatePrivateLinkJsonBody",
    "UploadInformation",
    "UploadInformationNeedsConversion",
    "UserInfoByIdResponse200",
    "UserInfoByIdResponse200Experiences",
    "UserInfoByIdResponse200NovelUserExperienceInfos",
    "UserInfoByIdResponse200TeamsItem",
    "UserListResponse200Item",
    "UserListResponse200ItemExperiences",
    "UserListResponse200ItemNovelUserExperienceInfos",
    "UserListResponse200ItemTeamsItem",
    "UserLoggedTimeResponse200",
    "UserLoggedTimeResponse200LoggedTimeItem",
    "UserLoggedTimeResponse200LoggedTimeItemPaymentInterval",
)



import json
import logging
from typing import TYPE_CHECKING, BinaryIO, Dict, List, Mapping, Optional, Tuple, Union

import attr
import httpx

from ..annotation import Annotation, AnnotationInfo
from ..client._generated.api.default import annotation_infos_by_task_id, task_info
from ..client.context import _get_generated_client
from ..dataset.dataset import RemoteDataset
from ..geometry import BoundingBox, Vec3Int
from ..utils import warn_deprecated
from .project import Project

logger = logging.getLogger(__name__)

if TYPE_CHECKING:
    from ..client._generated.models.task_info_response_200 import TaskInfoResponse200
    from ..client._generated.models.task_info_response_200_type import (
        TaskInfoResponse200Type,
    )
    from ..client._generated.models.task_infos_by_project_id_response_200_item import (
        TaskInfosByProjectIdResponse200Item,
    )
    from ..client._generated.models.task_infos_by_project_id_response_200_item_type import (
        TaskInfosByProjectIdResponse200ItemType,
    )


@attr.frozen
class TaskStatus:
    pending_instance_count: int
    active_instance_count: int
    finished_instance_count: int

    @property
    def open_instance_count(self) -> int:
        warn_deprecated("open_instance_count", "pending_instance_count")
        return self.pending_instance_count


@attr.frozen
class TaskType:
    task_type_id: str
    name: str
    description: str
    team_id: str
    team_name: str

    @classmethod
    def _from_generated_response(
        cls,
        response: Union[
            "TaskInfoResponse200Type", "TaskInfosByProjectIdResponse200ItemType"
        ],
    ) -> "TaskType":
        return cls(
            response.id,
            response.summary,
            response.description,
            response.team_id,
            response.team_name,
        )


@attr.frozen
class Task:
    """Data class containing information about a WEBKNOSSOS task"""

    task_id: str
    project_id: str
    dataset_name: str
    status: TaskStatus
    task_type: TaskType

    @classmethod
    def get_by_id(cls, task_id: str) -> "Task":
        """Returns the task specified by the passed id (if your token authorizes you to see it)"""
        client = _get_generated_client(enforce_auth=True)
        response = task_info.sync(id=task_id, client=client)
        assert (
            response is not None
        ), f"Requesting task infos from {client.base_url} failed."
        return cls._from_generated_response(response)

    @classmethod
    def create_from_annotations(
        cls,
        task_type_id: str,
        project_name: str,
        base_annotations: List[Annotation],
        needed_experience_domain: str,
        needed_experience_value: int,
        instances: int = 1,
        script_id: Optional[str] = None,
        bounding_box: Optional[BoundingBox] = None,
    ) -> List["Task"]:
        """Submits tasks in WEBKNOSSOS based on existing annotations, and returns the Task objects"""

        assert (
            len(base_annotations) > 0
        ), "Must supply at least one base annotation to create tasks"

        client = _get_generated_client(enforce_auth=True)
        url = f"{client.base_url}/api/tasks/createFromFiles"
        task_parameters = {
            "taskTypeId": task_type_id,
            "neededExperience": {
                "domain": needed_experience_domain,
                "value": needed_experience_value,
            },
            "pendingInstances": instances,
            "projectName": project_name,
            "scriptId": script_id,
            "boundingBox": bounding_box.to_wkw_dict()
            if bounding_box is not None
            else None,
        }
        form_data = {"formJSON": json.dumps(task_parameters)}
        files: Mapping[str, Tuple[str, Union[bytes, BinaryIO]]] = {
            f"{a.name}.zip": (f"{a.name}.zip", a._binary_zip())
            for a in base_annotations
        }

        response = httpx.post(
            url=url,
            headers=client.get_headers(),
            cookies=client.get_cookies(),
            timeout=client.get_timeout(),
            data=form_data,
            files=files,
        )
        assert (
            response.status_code == 200
        ), f"Failed to create tasks from files: {response.status_code}: {response.text}"

        return cls._handle_task_creation_response(response)

    @classmethod
    def create(
        cls,
        task_type_id: str,
        project_name: str,
        dataset_name: Union[str, RemoteDataset],
        needed_experience_domain: str,
        needed_experience_value: int,
        starting_position: Vec3Int,
        starting_rotation: Optional[Vec3Int] = Vec3Int(0, 0, 0),
        instances: int = 1,
        script_id: Optional[str] = None,
        bounding_box: Optional[BoundingBox] = None,
    ) -> List["Task"]:
        """Submits tasks in WEBKNOSSOS based on a dataset, starting position + rotation, and returns the Task objects"""

        client = _get_generated_client(enforce_auth=True)
        url = f"{client.base_url}/api/tasks"
        if isinstance(dataset_name, RemoteDataset):
            dataset_name = dataset_name._dataset_name
        task_parameters = {
            "taskTypeId": task_type_id,
            "neededExperience": {
                "domain": needed_experience_domain,
                "value": needed_experience_value,
            },
            "pendingInstances": instances,
            "projectName": project_name,
            "scriptId": script_id,
            "dataSet": dataset_name,
            "editPosition": starting_position,
            "editRotation": starting_rotation,
            "boundingBox": bounding_box.to_wkw_dict()
            if bounding_box is not None
            else None,
        }

        response = httpx.post(
            url=url,
            headers=client.get_headers(),
            cookies=client.get_cookies(),
            timeout=client.get_timeout(),
            json=[task_parameters],
        )
        assert (
            response.status_code == 200
        ), f"Failed to create tasks: {response.status_code}: {response.text}"

        return cls._handle_task_creation_response(response)

    @classmethod
    def _from_dict(cls, response_dict: Dict) -> "Task":
        from ..client._generated.models.task_info_response_200 import (
            TaskInfoResponse200,
        )

        return cls._from_generated_response(
            TaskInfoResponse200.from_dict(response_dict)
        )

    @classmethod
    def _from_generated_response(
        cls,
        response: Union["TaskInfoResponse200", "TaskInfosByProjectIdResponse200Item"],
    ) -> "Task":
        return cls(
            response.id,
            response.project_id,
            response.data_set,
            TaskStatus(
                response.status.pending,
                response.status.active,
                response.status.finished,
            ),
            TaskType._from_generated_response(response.type),
        )

    def get_annotation_infos(self) -> List[AnnotationInfo]:
        """Returns AnnotationInfo objects describing all task instances that have been started by annotators for this task"""
        client = _get_generated_client(enforce_auth=True)
        response = annotation_infos_by_task_id.sync(id=self.task_id, client=client)
        assert (
            response is not None
        ), f"Requesting annotation infos for task from {client.base_url} failed."
        return [AnnotationInfo._from_generated_response(a) for a in response]

    def get_project(self) -> Project:
        """Returns the project this task belongs to"""
        return Project.get_by_id(self.project_id)

    @classmethod
    def _handle_task_creation_response(cls, response: httpx.Response) -> List["Task"]:
        result = response.json()
        if "warnings" in result:
            warnings = result["warnings"]
            if len(warnings) > 0:
                logger.warning(
                    f"There were {len(warnings)} warnings during task creation:"
                )
            for warning in warnings:
                logger.warning(warning)
        assert "tasks" in result, "Invalid result of task creation"

        successes = []
        errors = []
        for t in result["tasks"]:
            print(t)
            if "success" in t:
                successes.append(t["success"])
            if "error" in t:
                errors.append(t["error"])
        if len(errors) > 0:
            logger.error(f"{len(errors)} tasks could not be created:")
            for error in errors:
                logger.error(error)
        if len(successes) > 0:
            logger.info(f"{len(successes)} tasks were successfully created.")
        return [cls._from_dict(t) for t in successes]



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.upload_information_needs_conversion import (
        UploadInformationNeedsConversion,
    )


T = TypeVar("T", bound="UploadInformation")


@attr.s(auto_attribs=True)
class UploadInformation:
    """
    Attributes:
        upload_id (str):
        needs_conversion (Union[Unset, UploadInformationNeedsConversion]):
    """

    upload_id: str
    needs_conversion: Union[Unset, "UploadInformationNeedsConversion"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        upload_id = self.upload_id
        needs_conversion: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.needs_conversion, Unset):
            needs_conversion = self.needs_conversion.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "uploadId": upload_id,
            }
        )
        if needs_conversion is not UNSET:
            field_dict["needsConversion"] = needs_conversion

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.upload_information_needs_conversion import (
            UploadInformationNeedsConversion,
        )

        d = src_dict.copy()
        upload_id = d.pop("uploadId")

        _needs_conversion = d.pop("needsConversion", UNSET)
        needs_conversion: Union[Unset, UploadInformationNeedsConversion]
        if isinstance(_needs_conversion, Unset):
            needs_conversion = UNSET
        else:
            needs_conversion = UploadInformationNeedsConversion.from_dict(
                _needs_conversion
            )

        upload_information = cls(
            upload_id=upload_id,
            needs_conversion=needs_conversion,
        )

        upload_information.additional_properties = d
        return upload_information

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="AnnotationInfoResponse200DataStore")


@attr.s(auto_attribs=True)
class AnnotationInfoResponse200DataStore:
    """
    Attributes:
        name (str):
        url (str):
        allows_upload (int):
        is_scratch (Union[Unset, int]):
    """

    name: str
    url: str
    allows_upload: int
    is_scratch: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        url = self.url
        allows_upload = self.allows_upload
        is_scratch = self.is_scratch

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "url": url,
                "allowsUpload": allows_upload,
            }
        )
        if is_scratch is not UNSET:
            field_dict["isScratch"] = is_scratch

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        url = d.pop("url")

        allows_upload = d.pop("allowsUpload")

        is_scratch = d.pop("isScratch", UNSET)

        annotation_info_response_200_data_store = cls(
            name=name,
            url=url,
            allows_upload=allows_upload,
            is_scratch=is_scratch,
        )

        annotation_info_response_200_data_store.additional_properties = d
        return annotation_info_response_200_data_store

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Iterable

import attr

if TYPE_CHECKING:
    from ..client._generated.models.folder_tree_response_200_item import (
        FolderTreeResponse200Item,
    )


def _get_folder_path(
    folder: "FolderTreeResponse200Item",
    all_folders: Iterable["FolderTreeResponse200Item"],
) -> str:
    if folder.parent is None:
        return folder.name
    else:
        return f"{_get_folder_path(next(f for f in all_folders if f.id == folder.parent), all_folders)}/{folder.name}"


@attr.frozen
class RemoteFolder:
    id: str
    name: str

    @classmethod
    def get_by_id(cls, folder_id: str) -> "RemoteFolder":
        from ..client._generated.api.default import folder_tree
        from ..client.context import _get_generated_client

        client = _get_generated_client(enforce_auth=True)

        response = folder_tree.sync(client=client)
        assert response is not None

        for folder_info in response:
            if folder_info.id == folder_id:
                return cls(name=folder_info.name, id=folder_info.id)

        raise KeyError(f"Could not find folder {folder_id}.")

    @classmethod
    def get_by_path(cls, path: str) -> "RemoteFolder":
        from ..client._generated.api.default import folder_tree
        from ..client.context import _get_generated_client

        client = _get_generated_client(enforce_auth=True)

        response = folder_tree.sync(client=client)
        assert response is not None

        for folder_info in response:
            folder_path = _get_folder_path(folder_info, response)
            if folder_path == path:
                return cls(name=folder_info.name, id=folder_info.id)

        raise KeyError(f"Could not find folder {path}.")



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="ShortLinkByKeyResponse200")


@attr.s(auto_attribs=True)
class ShortLinkByKeyResponse200:
    """
    Attributes:
        long_link (str):
        field_id (Union[Unset, str]):
        key (Union[Unset, str]):
    """

    long_link: str
    field_id: Union[Unset, str] = UNSET
    key: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        long_link = self.long_link
        field_id = self.field_id
        key = self.key

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "longLink": long_link,
            }
        )
        if field_id is not UNSET:
            field_dict["_id"] = field_id
        if key is not UNSET:
            field_dict["key"] = key

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        long_link = d.pop("longLink")

        field_id = d.pop("_id", UNSET)

        key = d.pop("key", UNSET)

        short_link_by_key_response_200 = cls(
            long_link=long_link,
            field_id=field_id,
            key=key,
        )

        short_link_by_key_response_200.additional_properties = d
        return short_link_by_key_response_200

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union, cast

import httpx

from ... import errors
from ...client import Client
from ...models.dataset_info_response_200 import DatasetInfoResponse200
from ...types import UNSET, Response, Unset


def _get_kwargs(
    organization_name: str,
    data_set_name: str,
    *,
    client: Client,
    sharing_token: Union[Unset, None, str] = UNSET,
) -> Dict[str, Any]:
    url = "{}/api/datasets/{organizationName}/{dataSetName}".format(
        client.base_url, organizationName=organization_name, dataSetName=data_set_name
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    params["sharingToken"] = sharing_token

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "follow_redirects": client.follow_redirects,
        "params": params,
    }


def _parse_response(
    *, client: Client, response: httpx.Response
) -> Optional[Union[Any, DatasetInfoResponse200]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = DatasetInfoResponse200.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.BAD_REQUEST:
        response_400 = cast(Any, None)
        return response_400
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Client, response: httpx.Response
) -> Response[Union[Any, DatasetInfoResponse200]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    organization_name: str,
    data_set_name: str,
    *,
    client: Client,
    sharing_token: Union[Unset, None, str] = UNSET,
) -> Response[Union[Any, DatasetInfoResponse200]]:
    """Get information about this dataset

    Args:
        organization_name (str):
        data_set_name (str):
        sharing_token (Union[Unset, None, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, DatasetInfoResponse200]]
    """

    kwargs = _get_kwargs(
        organization_name=organization_name,
        data_set_name=data_set_name,
        client=client,
        sharing_token=sharing_token,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    organization_name: str,
    data_set_name: str,
    *,
    client: Client,
    sharing_token: Union[Unset, None, str] = UNSET,
) -> Optional[Union[Any, DatasetInfoResponse200]]:
    """Get information about this dataset

    Args:
        organization_name (str):
        data_set_name (str):
        sharing_token (Union[Unset, None, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, DatasetInfoResponse200]
    """

    return sync_detailed(
        organization_name=organization_name,
        data_set_name=data_set_name,
        client=client,
        sharing_token=sharing_token,
    ).parsed


async def asyncio_detailed(
    organization_name: str,
    data_set_name: str,
    *,
    client: Client,
    sharing_token: Union[Unset, None, str] = UNSET,
) -> Response[Union[Any, DatasetInfoResponse200]]:
    """Get information about this dataset

    Args:
        organization_name (str):
        data_set_name (str):
        sharing_token (Union[Unset, None, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, DatasetInfoResponse200]]
    """

    kwargs = _get_kwargs(
        organization_name=organization_name,
        data_set_name=data_set_name,
        client=client,
        sharing_token=sharing_token,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    organization_name: str,
    data_set_name: str,
    *,
    client: Client,
    sharing_token: Union[Unset, None, str] = UNSET,
) -> Optional[Union[Any, DatasetInfoResponse200]]:
    """Get information about this dataset

    Args:
        organization_name (str):
        data_set_name (str):
        sharing_token (Union[Unset, None, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, DatasetInfoResponse200]
    """

    return (
        await asyncio_detailed(
            organization_name=organization_name,
            data_set_name=data_set_name,
            client=client,
            sharing_token=sharing_token,
        )
    ).parsed



import logging
from os import PathLike
from pathlib import Path
from typing import List, Optional, Tuple, TypeVar, Union, cast

import numpy as np
from rich.progress import track

from ..dataset import Dataset, LayerCategoryType
from ..dataset.properties import LayerViewConfiguration, dataset_converter
from ..geometry import BoundingBox, Mag, Vec3Int
from ._generated.api.datastore import dataset_download
from ._generated.api.default import dataset_info
from ._generated.types import Unset
from .context import _get_context

logger = logging.getLogger(__name__)


T = TypeVar("T")


_DOWNLOAD_CHUNK_SHAPE = Vec3Int(512, 512, 512)


def download_dataset(
    dataset_name: str,
    organization_id: str,
    sharing_token: Optional[str] = None,
    bbox: Optional[BoundingBox] = None,
    layers: Optional[List[str]] = None,
    mags: Optional[List[Mag]] = None,
    path: Optional[Union[PathLike, str]] = None,
    exist_ok: bool = False,
) -> Dataset:
    context = _get_context()
    client = context.generated_client

    dataset_info_response = dataset_info.sync_detailed(
        organization_name=organization_id,
        data_set_name=dataset_name,
        client=client,
        sharing_token=sharing_token,
    )
    assert dataset_info_response.status_code == 200, dataset_info_response
    parsed = dataset_info_response.parsed
    assert parsed is not None

    datastore_client = context.get_generated_datastore_client(parsed.data_store.url)
    optional_datastore_token = sharing_token or context.datastore_token

    actual_path = Path(dataset_name) if path is None else Path(path)
    if actual_path.exists():
        logger.warning(f"{actual_path} already exists, skipping download.")
        return Dataset.open(actual_path)

    data_layers = parsed.data_source.data_layers
    scale = parsed.data_source.scale
    if isinstance(data_layers, Unset) or isinstance(scale, Unset):
        raise RuntimeError(
            f"Could not download dataset {dataset_name}: {parsed.data_source.additional_properties.get('status', 'Unknown error.')}"
        )
    voxel_size = cast(Tuple[float, float, float], tuple(scale))
    dataset = Dataset(
        actual_path, name=parsed.name, voxel_size=voxel_size, exist_ok=exist_ok
    )
    for layer_name in layers or [i.name for i in data_layers]:
        response_layers = [i for i in data_layers if i.name == layer_name]
        assert (
            len(response_layers) > 0
        ), f"The provided layer name {layer_name} could not be found in the requested dataset."
        assert (
            len(response_layers) == 1
        ), f"The provided layer name {layer_name} was found multiple times in the requested dataset."
        response_layer = response_layers[0]
        category = cast(LayerCategoryType, response_layer.category)
        layer = dataset.add_layer(
            layer_name=layer_name,
            category=category,
            dtype_per_layer=response_layer.element_class,
            num_channels=3 if response_layer.element_class == "uint24" else 1,
            largest_segment_id=response_layer.additional_properties.get(
                "largestSegmentId", None
            ),
        )

        default_view_configuration_dict = None
        if not isinstance(response_layer.default_view_configuration, Unset):
            default_view_configuration_dict = (
                response_layer.default_view_configuration.to_dict()
            )

        if default_view_configuration_dict is not None:
            default_view_configuration = dataset_converter.structure(
                default_view_configuration_dict, LayerViewConfiguration
            )
            layer.default_view_configuration = default_view_configuration

        if bbox is None:
            response_bbox = response_layer.bounding_box
            layer.bounding_box = BoundingBox(
                response_bbox.top_left,
                (response_bbox.width, response_bbox.height, response_bbox.depth),
            )
        else:
            assert isinstance(
                bbox, BoundingBox
            ), f"Expected a BoundingBox object for the bbox parameter but got {type(bbox)}"
            layer.bounding_box = bbox
        if mags is None:
            mags = [Mag(mag) for mag in response_layer.resolutions]
        for mag in mags:
            mag_view = layer.get_or_add_mag(
                mag,
                compress=True,
                chunk_shape=Vec3Int.full(32),
                chunks_per_shard=_DOWNLOAD_CHUNK_SHAPE // 32,
            )
            aligned_bbox = layer.bounding_box.align_with_mag(mag, ceil=True)
            download_chunk_shape_in_mag = _DOWNLOAD_CHUNK_SHAPE * mag.to_vec3_int()
            for chunk in track(
                list(
                    aligned_bbox.chunk(
                        download_chunk_shape_in_mag, download_chunk_shape_in_mag
                    )
                ),
                description=f"Downloading layer={layer.name} mag={mag}",
            ):
                chunk_in_mag = chunk.in_mag(mag)
                response = dataset_download.sync_detailed(
                    organization_name=organization_id,
                    data_set_name=dataset_name,
                    data_layer_name=layer_name,
                    mag=mag.to_long_layer_name(),
                    client=datastore_client,
                    token=optional_datastore_token,
                    x=chunk.topleft.x,
                    y=chunk.topleft.y,
                    z=chunk.topleft.z,
                    width=chunk_in_mag.size.x,
                    height=chunk_in_mag.size.y,
                    depth=chunk_in_mag.size.z,
                )
                assert response.status_code == 200, response
                assert (
                    response.headers["missing-buckets"] == "[]"
                ), f"Download contained missing buckets {response.headers['missing-buckets']}."
                data = np.frombuffer(
                    response.content, dtype=layer.dtype_per_channel
                ).reshape(layer.num_channels, *chunk_in_mag.size, order="F")
                mag_view.write(data, absolute_offset=chunk.topleft)
    return dataset



from http import HTTPStatus
from typing import Any, Dict, Optional

import httpx

from ... import errors
from ...client import Client
from ...models.task_create_from_files_json_body import TaskCreateFromFilesJsonBody
from ...types import Response


def _get_kwargs(
    *,
    client: Client,
    json_body: TaskCreateFromFilesJsonBody,
) -> Dict[str, Any]:
    url = "{}/api/tasks/createFromFiles".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    json_json_body = json_body.to_dict()

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "follow_redirects": client.follow_redirects,
        "json": json_json_body,
    }


def _parse_response(*, client: Client, response: httpx.Response) -> Optional[Any]:
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Client, response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Client,
    json_body: TaskCreateFromFilesJsonBody,
) -> Response[Any]:
    """Create new tasks from existing annotation files
    Expects:
     - As Form data:
       - taskTypeId (string) id of the task type to be used for the new tasks
       - neededExperience (Experience) experience domain and level that selects which users can get the
    new tasks
       - pendingInstances (int) if greater than one, multiple instances of the task will be given to
    users to annotate
       - projectName (string) name of the project the task should be part of
       - scriptId (string, optional) id of a user script that should be loaded for the annotators of the
    new tasks
       - boundingBox (BoundingBox, optional) limit the bounding box where the annotators should be
    active
     - As File attachment
       - A zip file containing base annotations (each either NML or zip with NML + volume) for the new
    tasks. One task will be created per annotation.

    Args:
        json_body (TaskCreateFromFilesJsonBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(client=client, response=response)


async def asyncio_detailed(
    *,
    client: Client,
    json_body: TaskCreateFromFilesJsonBody,
) -> Response[Any]:
    """Create new tasks from existing annotation files
    Expects:
     - As Form data:
       - taskTypeId (string) id of the task type to be used for the new tasks
       - neededExperience (Experience) experience domain and level that selects which users can get the
    new tasks
       - pendingInstances (int) if greater than one, multiple instances of the task will be given to
    users to annotate
       - projectName (string) name of the project the task should be part of
       - scriptId (string, optional) id of a user script that should be loaded for the annotators of the
    new tasks
       - boundingBox (BoundingBox, optional) limit the bounding box where the annotators should be
    active
     - As File attachment
       - A zip file containing base annotations (each either NML or zip with NML + volume) for the new
    tasks. One task will be created per annotation.

    Args:
        json_body (TaskCreateFromFilesJsonBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(client=client, response=response)
